"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

__all__ = (
    "DisplayAsTypes",
    "Distortions",
    "FFA_AberrationTypes",
    "FFA_DecompositionTypes",
    "FFA_DisplayTypes",
    "FFA_FieldShapes",
    "FFA_ShowAsTypes",
    "FieldScanDirections",
    "RayTraceType",
    "ZernikeCoefficientTypes",
)

class DisplayAsTypes(Enum):
    Percent = 0
    Absolute = 1

class Distortions(Enum):
    F_TanTheta = 0
    F_Theta = 1
    Cal_F_Theta = 2
    Cal_F_TanTheta = 3
    SMIA_TV = 4

class FFA_AberrationTypes(Enum):
    Defocus = 0
    PrimaryAstigmatism = 1
    PrimaryComa = 2
    SpecifiedTerm = 3

class FFA_DecompositionTypes(Enum):
    ZernikeTerms = 0

class FFA_DisplayTypes(Enum):
    Absolute = 0
    Relative = 1
    Average = 2

class FFA_FieldShapes(Enum):
    Rectangular = 0
    Elliptical = 1

class FFA_ShowAsTypes(Enum):
    GreyScale = 0
    GreyScaleInverted = 1
    FalseColor = 2
    FalseColorInverted = 3
    Icons = 4

class FieldScanDirections(Enum):
    Plus_Y = 0
    Plus_X = 1
    Minus_Y = 2
    Minus_X = 3

class RayTraceType(Enum):
    DirectionCosines = 0
    TangentAngle = 1
    YmUmYcUc = 2

class ZernikeCoefficientTypes(Enum):
    Fringe = 0
    Standard = 1
    Annular = 2



"""Zemax OpticStudio analyses from the Surface category."""

from __future__ import annotations

import pandas as pd

from zospy.analyses.base import AnalysisResult, AttrDict, OnComplete, new_analysis
from zospy.api import constants
from zospy.utils import zputils
from zospy.zpcore import OpticStudioSystem


def curvature(
    oss: OpticStudioSystem,
    sampling: str = "65x65",
    data: constants.Analysis.SurfaceCurvatureData | str = "TangentialCurvature",
    remove: constants.Analysis.RemoveOptions | str | None = "None_",
    surface: int = 1,
    showas: constants.Analysis.ShowAs | str = "Contour",
    offaxiscoordinates: bool = False,
    contourformat: str = "",
    bfs_criterion: constants.Analysis.BestFitSphereOptions | str = "MinimumVolume",
    bfs_reversedirection: bool = False,
    oncomplete: OnComplete | str = OnComplete.Close,
) -> AnalysisResult:
    """Wrapper around the OpticStudio Curvature analysis.

    Parameters
    ----------
    oss: zospy.core.OpticStudioSystem
        A ZOSPy OpticStudioSystem instance. Should be sequential.
    sampling: str | int
        The size of the used grid, either string (e.g. '65x65') or int. The integer will be treated as if obtained from
        zospy.constants.Analysis.SampleSizes_Pow2Plus1_X. Defaults to '65x65'.
    data: str
        The used data type. Should be one of ['TangentialCurvature', 'SagittalCurvature', 'X_Curvature', 'Y_Curvature']
        or int. The integer will be treated as if obtained from zospy.constants.Analysis.SurfaceCurvatureData. Defaults
        to 'TangentialCurvature'.
    remove: str | int
        Defines whether a reference volume is removed or not. Should be one of ['None', 'BaseROC', 'BestFitSphere'] or
        int. The integer will be treated as if obtained from zospy.constants.Analysis.RemoveOptions. Defaults
        to 'None'.
    surface: int
        The surface that is te be analyzed. defaults to 1.
    showas: str | int
        Defines how the data is displayed in OpticStudio. Should be one of ['Surface', 'Contour', 'GreyScale',
        'InverseGreyScale', 'FalseColor', 'InverseFalseColor'] or int. The integer will be treated as if obtained from
        zospyconstants.Analysis.ShowAs. Defaults to 'Contour'.
    offaxiscoordinates: bool
        Defines whether apertures defined in the Surface Properties of the surface are considered or not. Defaults to
        False.
    contourformat: str
        The contour format. Only usable when showas == 'Contour'. Defaults to ''.
    bfs_criterion: str | int
        The criterion for BFS removal. Only usable when remove == 'BestFitSphere'. Should be one of ['MinimumVolume',
        'MinimumRMS', 'MinimumRMSWithOffset'] or int. The integer will be treated as if obtained from
        constants.Analysis.BestFitSphereOptions. Defaults to 'MinimumVolume'.
    bfs_reversedirection: bool
        Defines if the sign of the BFS radius should be reversed or not. Only usable when remove == 'BestFitSphere'
        and bfs_criterion == 'MinimumVolume'. Defaults to False.
    oncomplete: OnComplete | str
        Defines behaviour upon completion of the analysis. Should be one of ['Close', 'Release', 'Sustain']. If 'Close',
        the analysis will be closed after completion. If 'Release', the analysis will remain open in OpticStudio, but
        the link with python will be destroyed. If 'Sustain' the analysis will be kept open in OpticStudio and the link
        with python will be sustained. To enable interaction when oncomplete == 'Sustain', the OpticStudio Analysis
        instance will be available in the returned AnalysisResult through AnalysisResult.Analysis. Defaults to 'Close'.

    Returns
    -------
    AnalysisResult
        A SurfaceCurvature analysis result
    """
    analysis_type = constants.Analysis.AnalysisIDM.SurfaceCurvature

    analysis = new_analysis(oss, analysis_type)

    # Apply settings
    analysis.Settings.Sampling = getattr(
        constants.Analysis.SampleSizes_Pow2Plus1_X, zputils.standardize_sampling(sampling)
    )
    analysis.Settings.Data = constants.process_constant(constants.Analysis.SurfaceCurvatureData, data)
    analysis.Settings.RemoveOption = constants.process_constant(constants.Analysis.RemoveOptions, remove)
    analysis.set_surface(surface)
    analysis.Settings.ShowAs = constants.process_constant(constants.Analysis.ShowAs, showas)

    analysis.Settings.ConsiderOffAxisAperture = offaxiscoordinates
    if analysis.Settings.ShowAs == constants.Analysis.ShowAs.Contour:  # ContourFormat becomes available
        analysis.Settings.ContourFormat = contourformat

    if analysis.Settings.RemoveOption == constants.Analysis.RemoveOptions.BestFitSphere:  # Add Best fit sphere options
        analysis.Settings.BestFitSphereOption = constants.process_constant(
            constants.Analysis.BestFitSphereOptions, bfs_criterion
        )

        if analysis.Settings.BestFitSphereOption == constants.Analysis.BestFitSphereOptions.MinimumVolume:
            # Reversed direction becomes available
            analysis.Settings.ReverseDirection = bfs_reversedirection

    # Calculate
    analysis.ApplyAndWaitForCompletion()

    # Get headerdata, metadata and messages
    headerdata = analysis.get_header_data()
    metadata = analysis.get_metadata()
    messages = analysis.get_messages()

    # Get settings
    settings = pd.Series(name="Settings", dtype=object)

    settings.loc["Sampling"] = str(analysis.Settings.Sampling)
    settings.loc["Data"] = str(analysis.Settings.Data)
    settings.loc["RemoveOption"] = str(analysis.Settings.RemoveOption)
    settings.loc["Surface"] = analysis.Settings.Surface.GetSurfaceNumber()
    settings.loc["ShowAs"] = str(analysis.Settings.ShowAs)
    settings.loc["ConsiderOffAxisAperture"] = analysis.Settings.ConsiderOffAxisAperture

    if analysis.Settings.ShowAs == constants.Analysis.ShowAs.Contour:  # ContourFormat is available
        settings.loc["Contourformat"] = str(analysis.Settings.ContourFormat)

    if analysis.Settings.RemoveOption == constants.Analysis.RemoveOptions.BestFitSphere:  # Add Best fit sphere options
        settings.loc["BestFitSphereOptions"] = str(analysis.Settings.BestFitSphereOption)
        if analysis.Settings.BestFitSphereOption == constants.Analysis.BestFitSphereOptions.MinimumVolume:
            # Reversed direction is available
            settings.loc["ReverseDirection"] = analysis.Settings.ReverseDirection

    # Get data
    if analysis.Results.NumberOfDataGrids <= 0:
        data = None
    elif analysis.Results.NumberOfDataGrids == 1:
        data = zputils.unpack_datagrid(analysis.Results.DataGrids[0])
    else:
        data = AttrDict()
        for ii in range(analysis.Results.NumberOfDataGrids):
            desc = analysis.Results.DataGrids[ii].Description
            key = desc if desc != "" else str(ii)
            data[key] = zputils.unpack_datagrid(analysis.Results.DataGrids[ii])

    result = AnalysisResult(
        analysistype=str(analysis_type),
        data=data,
        settings=settings,
        metadata=metadata,
        headerdata=headerdata,
        messages=messages,
    )

    return analysis.complete(oncomplete, result)



"""
Utility functions for ZOSPy.

`zospy.functions` contains utility functions for zospy. These functions are available through its submodules:

- **`zospy.functions.lde`** provides helper functions for the Lens Data Editor (LDE);
- **`zospy.functions.nce`** provides helper functions for the Non-sequential Component Editor (NCE).
"""

from . import lde, nce

__all__ = ("lde", "nce")



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

__all__ = ("NSCSagRemoveOptions", "NSCSagShowAs")

class NSCSagRemoveOptions(Enum):
    None_ = 0
    BaseROC = 1
    BestFitSphere = 2
    AverageSag = 3
    MinimumSag = 4

class NSCSagShowAs(Enum):
    Surface = 0
    Contour = 1
    GreyScale = 2
    InverseGreyScale = 3
    FalseColor = 4
    InverseFalseColor = 5
    X_CrossSection = 6
    Y_CrossSection = 7



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = (
    "DateTimeType",
    "EncodingType",
    "IPreferencesFolders",
    "IPreferencesFolders",
    "IPreferencesGeneral",
    "IPreferencesGeneral",
    "LanguageType",
    "ShowLineAsType",
)

class DateTimeType:
    # None = 0
    Date = 1
    DateTime = 2

class EncodingType:
    ANSI = 0
    Unicode = 1

class IPreferencesFolders:
    def ChangeDataDirectory(self, newDirectory: str) -> None: ...
    @property
    def CoatingDirectory(self) -> str: ...
    @property
    def CreoDirectory(self) -> str: ...
    @property
    def CreoInstallDirectory(self) -> str: ...
    @property
    def DataDirectory(self) -> str: ...
    @property
    def GlassDirectory(self) -> str: ...
    @property
    def ImagesDirectory(self) -> str: ...
    @property
    def InventorDirectory(self) -> str: ...
    @property
    def LensDirectory(self) -> str: ...
    @property
    def MatlabDirectory(self) -> str: ...
    @property
    def ObjectsDirectory(self) -> str: ...
    @property
    def PopDirectory(self) -> str: ...
    @property
    def ProgramDirectory(self) -> str: ...
    @property
    def ScatterDirectory(self) -> str: ...
    @property
    def SolidWorksDirectory(self) -> str: ...
    @property
    def UndoDirectory(self) -> str: ...
    @property
    def ZplDirectory(self) -> str: ...
    @CoatingDirectory.setter
    def CoatingDirectory(self, value: str) -> None: ...
    @CreoDirectory.setter
    def CreoDirectory(self, value: str) -> None: ...
    @CreoInstallDirectory.setter
    def CreoInstallDirectory(self, value: str) -> None: ...
    @GlassDirectory.setter
    def GlassDirectory(self, value: str) -> None: ...
    @ImagesDirectory.setter
    def ImagesDirectory(self, value: str) -> None: ...
    @InventorDirectory.setter
    def InventorDirectory(self, value: str) -> None: ...
    @LensDirectory.setter
    def LensDirectory(self, value: str) -> None: ...
    @MatlabDirectory.setter
    def MatlabDirectory(self, value: str) -> None: ...
    @ObjectsDirectory.setter
    def ObjectsDirectory(self, value: str) -> None: ...
    @PopDirectory.setter
    def PopDirectory(self, value: str) -> None: ...
    @ScatterDirectory.setter
    def ScatterDirectory(self, value: str) -> None: ...
    @SolidWorksDirectory.setter
    def SolidWorksDirectory(self, value: str) -> None: ...
    @UndoDirectory.setter
    def UndoDirectory(self, value: str) -> None: ...
    @ZplDirectory.setter
    def ZplDirectory(self, value: str) -> None: ...

class IPreferencesGeneral:
    @property
    def DateTimeFormat(self) -> DateTimeType: ...
    @property
    def IncludeCalculatedDataInSession(self) -> bool: ...
    @property
    def IncludeToleranceDataInSession(self) -> bool: ...
    @property
    def Language(self) -> LanguageType: ...
    @property
    def TXTFileEncoding(self) -> EncodingType: ...
    @property
    def UpdateMostRecentlyUsedList(self) -> bool: ...
    @property
    def UserPreferences(self) -> str: ...
    @property
    def UseSessionFiles(self) -> bool: ...
    @property
    def ZMXFileEncoding(self) -> EncodingType: ...
    def GetIsGPUEnabled(self) -> bool: ...
    def GetUseACIS(self) -> bool: ...
    @DateTimeFormat.setter
    def DateTimeFormat(self, value: DateTimeType) -> None: ...
    @IncludeCalculatedDataInSession.setter
    def IncludeCalculatedDataInSession(self, value: bool) -> None: ...
    @IncludeToleranceDataInSession.setter
    def IncludeToleranceDataInSession(self, value: bool) -> None: ...
    @Language.setter
    def Language(self, value: LanguageType) -> None: ...
    @TXTFileEncoding.setter
    def TXTFileEncoding(self, value: EncodingType) -> None: ...
    @UpdateMostRecentlyUsedList.setter
    def UpdateMostRecentlyUsedList(self, value: bool) -> None: ...
    @UserPreferences.setter
    def UserPreferences(self, value: str) -> None: ...
    @UseSessionFiles.setter
    def UseSessionFiles(self, value: bool) -> None: ...
    @ZMXFileEncoding.setter
    def ZMXFileEncoding(self, value: EncodingType) -> None: ...
    def SetIsGPUEnabled(self, isEnabled: bool) -> bool: ...
    def SetUseACIS(self, use: bool) -> bool: ...

class LanguageType:
    Chinese = 0
    English = 1
    Japanese = 4

class ShowLineAsType:
    TextAbove = 0
    FileName = 1
    ConfigurationNumber = 2



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

__all__ = (
    "FieldColumn",
    "FieldNormalizationType",
    "FieldPattern",
    "FieldType",
    "FNumberComputationType",
    "HuygensIntegralSettings",
    "ParaxialRaysSetting",
    "PolarizationMethod",
    "QuadratureSteps",
    "RayAimingMethod",
    "ReferenceOPDSetting",
    "WavelengthPreset",
    "ZemaxAfocalModeUnits",
    "ZemaxAnalysisUnits",
    "ZemaxApertureType",
    "ZemaxApodizationType",
    "ZemaxMTFUnits",
    "ZemaxSourceUnits",
    "ZemaxSystemUnits",
    "ZemaxUnitPrefix",
)

class FieldColumn(Enum):
    Comment = 0
    X = 1
    Y = 2
    Weight = 3
    VDX = 4
    VDY = 5
    VCX = 6
    VCY = 7
    TAN = 8
    VAN = 8

class FieldNormalizationType(Enum):
    Radial = 0
    Rectangular = 1

class FieldPattern(Enum):
    UniformY = 0
    EqualAreaY = 1
    UniformX = 2
    EqualAreaX = 3
    Grid = 4
    UniformRadial = 5
    EqualAreaRadial = 6

class FieldType(Enum):
    Angle = 0
    ObjectHeight = 1
    ParaxialImageHeight = 2
    RealImageHeight = 3
    TheodoliteAngle = 4

class FNumberComputationType(Enum):
    TracingRays = 0
    PupilSizePosition = 1

class HuygensIntegralSettings(Enum):
    Auto = 0
    Planar = 1
    Spherical = 2

class ParaxialRaysSetting(Enum):
    ConsiderCoordinateBreaks = 0
    IgnoreCoordinateBreaks = 1

class PolarizationMethod(Enum):
    XAxisMethod = 0
    YAxisMethod = 1
    ZAxisMethod = 2

class QuadratureSteps(Enum):
    S2 = 0
    S4 = 1
    S6 = 2
    S8 = 3
    S10 = 4
    S12 = 5

class RayAimingMethod(Enum):
    Off = 0
    Paraxial = 1
    Real = 2

class ReferenceOPDSetting(Enum):
    Absolute = 0
    Infinity = 1
    ExitPupil = 2
    Absolute2 = 3

class WavelengthPreset(Enum):
    FdC_Visible = 0
    Photopic_Bright = 1
    Scotopic_Dark = 2
    HeNe_0p6328 = 3
    HeNe_0p5438 = 4
    Argon_0p4880 = 5
    Argon_0p5145 = 6
    NDYAG_1p0641 = 7
    NDGlass_1p054 = 8
    CO2_10p60 = 9
    CrLiSAF_0p840 = 10
    TiAl203_0p760 = 11
    Ruby_0p6943 = 12
    HeCadmium_0p4416 = 13
    HeCadmium_0p3536 = 14
    HeCadmium_0p3250 = 15
    t_1p014 = 16
    r_0p707 = 17
    C_0p656 = 18
    d_0p587 = 19
    F_0p486 = 20
    g_0p436 = 21
    i_0p365 = 22
    Fp_0p365 = 23
    e_0p54607 = 24
    Cp_0p6438469 = 25
    FpeCp_Visible = 26
    THz_193p10 = 27

class ZemaxAfocalModeUnits(Enum):
    Microradians = 0
    Milliradians = 1
    Radians = 2
    ArcSeconds = 3
    ArcMinutes = 4
    Degrees = 5

class ZemaxAnalysisUnits(Enum):
    WattsPerMMSq = 0
    WattsPerCMSq = 1
    WattsPerinSq = 2
    WattsPerMSq = 3
    WattsPerftSq = 4

class ZemaxApertureType(Enum):
    EntrancePupilDiameter = 0
    ImageSpaceFNum = 1
    ObjectSpaceNA = 2
    FloatByStopSize = 3
    ParaxialWorkingFNum = 4
    ObjectConeAngle = 5

class ZemaxApodizationType(Enum):
    Uniform = 0
    Gaussian = 1
    CosineCubed = 2

class ZemaxMTFUnits(Enum):
    CyclesPerMillimeter = 0
    CyclesPerMilliradian = 1

class ZemaxSourceUnits(Enum):
    Watts = 0
    Lumens = 1
    Joules = 2

class ZemaxSystemUnits(Enum):
    Millimeters = 0
    Centimeters = 1
    Inches = 2
    Meters = 3

class ZemaxUnitPrefix(Enum):
    Femto = 0
    Pico = 1
    Nano = 2
    Micro = 3
    Milli = 4
    None_ = 5
    Kilo = 6
    Mega = 7
    Giga = 8
    Tera = 9



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from zospy.api._ZOSAPI.Analysis import (
    HuygensSurfaceMftShowAsTypes,
    SampleSizes,
    SampleSizes_ContrastLoss,
    ShowAs,
)
from zospy.api._ZOSAPI.Analysis.Settings import (
    IAS_,
    IAS_Detector,
    IAS_Field,
    IAS_Surface,
    IAS_Wavelength,
    Rotations,
    ScanTypes,
)

__all__ = (
    "HuygensMtfTypes",
    "IAS_ContrastLossMap",
    "IAS_FftMtf",
    "IAS_FftMtfMap",
    "IAS_FftMtfvsField",
    "IAS_FftSurfaceMtf",
    "IAS_FftThroughFocusMtf",
    "IAS_GeometricMtf",
    "IAS_GeometricMtfMap",
    "IAS_GeometricMtfvsField",
    "IAS_GeometricThroughFocusMtf",
    "IAS_HuygensMtf",
    "IAS_HuygensMtfvsField",
    "IAS_HuygensSurfaceMtf",
    "IAS_HuygensThroughFocusMtf",
    "IAS_NSCGeometricMtf",
    "MtfDataTypes",
    "MtfTypes",
    "ShowAsTypes",
    "SurfaceMtfTypes",
)

class HuygensMtfTypes:
    Modulation = 0

class IAS_ContrastLossMap(IAS_):
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def Frequency(self) -> float: ...
    @property
    def Normalize(self) -> bool: ...
    @property
    def SampleSize(self) -> SampleSizes_ContrastLoss: ...
    @property
    def ShowOPD(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @Frequency.setter
    def Frequency(self, value: float) -> None: ...
    @Normalize.setter
    def Normalize(self, value: bool) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes_ContrastLoss) -> None: ...
    @ShowOPD.setter
    def ShowOPD(self, value: bool) -> None: ...

class IAS_FftMtf(IAS_):
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def MaximumFrequency(self) -> float: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def ShowDiffractionLimit(self) -> bool: ...
    @property
    def Surface(self) -> IAS_Surface: ...
    @property
    def Type(self) -> MtfTypes: ...
    @property
    def UseDashes(self) -> bool: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @MaximumFrequency.setter
    def MaximumFrequency(self, value: float) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @ShowDiffractionLimit.setter
    def ShowDiffractionLimit(self, value: bool) -> None: ...
    @Type.setter
    def Type(self, value: MtfTypes) -> None: ...
    @UseDashes.setter
    def UseDashes(self, value: bool) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_FftMtfMap(IAS_):
    @property
    def Frequency(self) -> float: ...
    @property
    def MTF_DataType(self) -> MtfDataTypes: ...
    @property
    def ReferenceField(self) -> IAS_Field: ...
    @property
    def RemoveVignetting(self) -> bool: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def ShowAs(self) -> ShowAsTypes: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @property
    def X_Field_Width(self) -> float: ...
    @property
    def X_Pixels(self) -> int: ...
    @property
    def Y_Field_Width(self) -> float: ...
    @property
    def Y_Pixels(self) -> int: ...
    @Frequency.setter
    def Frequency(self, value: float) -> None: ...
    @MTF_DataType.setter
    def MTF_DataType(self, value: MtfDataTypes) -> None: ...
    @RemoveVignetting.setter
    def RemoveVignetting(self, value: bool) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @ShowAs.setter
    def ShowAs(self, value: ShowAsTypes) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...
    @X_Field_Width.setter
    def X_Field_Width(self, value: float) -> None: ...
    @X_Pixels.setter
    def X_Pixels(self, value: int) -> None: ...
    @Y_Field_Width.setter
    def Y_Field_Width(self, value: float) -> None: ...
    @Y_Pixels.setter
    def Y_Pixels(self, value: int) -> None: ...

class IAS_FftMtfvsField(IAS_):
    @property
    def FieldDensity(self) -> int: ...
    @property
    def Freq_1(self) -> float: ...
    @property
    def Freq_2(self) -> float: ...
    @property
    def Freq_3(self) -> float: ...
    @property
    def Freq_4(self) -> float: ...
    @property
    def Freq_5(self) -> float: ...
    @property
    def Freq_6(self) -> float: ...
    @property
    def RemoveVignetting(self) -> bool: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def ScanType(self) -> ScanTypes: ...
    @property
    def UseDashes(self) -> bool: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @FieldDensity.setter
    def FieldDensity(self, value: int) -> None: ...
    @Freq_1.setter
    def Freq_1(self, value: float) -> None: ...
    @Freq_2.setter
    def Freq_2(self, value: float) -> None: ...
    @Freq_3.setter
    def Freq_3(self, value: float) -> None: ...
    @Freq_4.setter
    def Freq_4(self, value: float) -> None: ...
    @Freq_5.setter
    def Freq_5(self, value: float) -> None: ...
    @Freq_6.setter
    def Freq_6(self, value: float) -> None: ...
    @RemoveVignetting.setter
    def RemoveVignetting(self, value: bool) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @ScanType.setter
    def ScanType(self, value: ScanTypes) -> None: ...
    @UseDashes.setter
    def UseDashes(self, value: bool) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_FftSurfaceMtf(IAS_):
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def Rotation(self) -> Rotations: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def ScaleFactor(self) -> float: ...
    @property
    def ShowAs(self) -> ShowAs: ...
    @property
    def Type(self) -> SurfaceMtfTypes: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @Rotation.setter
    def Rotation(self, value: Rotations) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @ScaleFactor.setter
    def ScaleFactor(self, value: float) -> None: ...
    @ShowAs.setter
    def ShowAs(self, value: ShowAs) -> None: ...
    @Type.setter
    def Type(self, value: SurfaceMtfTypes) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_FftThroughFocusMtf(IAS_):
    @property
    def DeltaFocus(self) -> float: ...
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def Frequency(self) -> float: ...
    @property
    def NumberOfSteps(self) -> int: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def Type(self) -> MtfTypes: ...
    @property
    def UseDashes(self) -> bool: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @DeltaFocus.setter
    def DeltaFocus(self, value: float) -> None: ...
    @Frequency.setter
    def Frequency(self, value: float) -> None: ...
    @NumberOfSteps.setter
    def NumberOfSteps(self, value: int) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @Type.setter
    def Type(self, value: MtfTypes) -> None: ...
    @UseDashes.setter
    def UseDashes(self, value: bool) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_GeometricMtf(IAS_):
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def MaximumFrequency(self) -> float: ...
    @property
    def MultiplyByDiffractionLimit(self) -> bool: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def ScatterRays(self) -> bool: ...
    @property
    def UseDashes(self) -> bool: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @MaximumFrequency.setter
    def MaximumFrequency(self, value: float) -> None: ...
    @MultiplyByDiffractionLimit.setter
    def MultiplyByDiffractionLimit(self, value: bool) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @ScatterRays.setter
    def ScatterRays(self, value: bool) -> None: ...
    @UseDashes.setter
    def UseDashes(self, value: bool) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_GeometricMtfMap(IAS_):
    @property
    def Frequency(self) -> float: ...
    @property
    def MTF_DataType(self) -> MtfDataTypes: ...
    @property
    def ReferenceField(self) -> IAS_Field: ...
    @property
    def RemoveVignetting(self) -> bool: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def ScatterRays(self) -> bool: ...
    @property
    def ShowAs(self) -> ShowAsTypes: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @property
    def X_Field_Width(self) -> float: ...
    @property
    def X_Pixels(self) -> int: ...
    @property
    def Y_Field_Width(self) -> float: ...
    @property
    def Y_Pixels(self) -> int: ...
    @Frequency.setter
    def Frequency(self, value: float) -> None: ...
    @MTF_DataType.setter
    def MTF_DataType(self, value: MtfDataTypes) -> None: ...
    @RemoveVignetting.setter
    def RemoveVignetting(self, value: bool) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @ScatterRays.setter
    def ScatterRays(self, value: bool) -> None: ...
    @ShowAs.setter
    def ShowAs(self, value: ShowAsTypes) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...
    @X_Field_Width.setter
    def X_Field_Width(self, value: float) -> None: ...
    @X_Pixels.setter
    def X_Pixels(self, value: int) -> None: ...
    @Y_Field_Width.setter
    def Y_Field_Width(self, value: float) -> None: ...
    @Y_Pixels.setter
    def Y_Pixels(self, value: int) -> None: ...

class IAS_GeometricMtfvsField(IAS_):
    @property
    def FieldDensity(self) -> int: ...
    @property
    def Freq_1(self) -> float: ...
    @property
    def Freq_2(self) -> float: ...
    @property
    def Freq_3(self) -> float: ...
    @property
    def Freq_4(self) -> float: ...
    @property
    def Freq_5(self) -> float: ...
    @property
    def Freq_6(self) -> float: ...
    @property
    def MultiplyByDiffractionLimit(self) -> bool: ...
    @property
    def RemoveVignetting(self) -> bool: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def ScanType(self) -> ScanTypes: ...
    @property
    def ScatterRays(self) -> bool: ...
    @property
    def UseDashes(self) -> bool: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @FieldDensity.setter
    def FieldDensity(self, value: int) -> None: ...
    @Freq_1.setter
    def Freq_1(self, value: float) -> None: ...
    @Freq_2.setter
    def Freq_2(self, value: float) -> None: ...
    @Freq_3.setter
    def Freq_3(self, value: float) -> None: ...
    @Freq_4.setter
    def Freq_4(self, value: float) -> None: ...
    @Freq_5.setter
    def Freq_5(self, value: float) -> None: ...
    @Freq_6.setter
    def Freq_6(self, value: float) -> None: ...
    @MultiplyByDiffractionLimit.setter
    def MultiplyByDiffractionLimit(self, value: bool) -> None: ...
    @RemoveVignetting.setter
    def RemoveVignetting(self, value: bool) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @ScanType.setter
    def ScanType(self, value: ScanTypes) -> None: ...
    @ScatterRays.setter
    def ScatterRays(self, value: bool) -> None: ...
    @UseDashes.setter
    def UseDashes(self, value: bool) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_GeometricThroughFocusMtf(IAS_):
    @property
    def DeltaFocus(self) -> float: ...
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def Frequency(self) -> float: ...
    @property
    def MultiplyByDiffractionLimit(self) -> bool: ...
    @property
    def NumberOfSteps(self) -> int: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def ScatterRays(self) -> bool: ...
    @property
    def UseDashes(self) -> bool: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @DeltaFocus.setter
    def DeltaFocus(self, value: float) -> None: ...
    @Frequency.setter
    def Frequency(self, value: float) -> None: ...
    @MultiplyByDiffractionLimit.setter
    def MultiplyByDiffractionLimit(self, value: bool) -> None: ...
    @NumberOfSteps.setter
    def NumberOfSteps(self, value: int) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @ScatterRays.setter
    def ScatterRays(self, value: bool) -> None: ...
    @UseDashes.setter
    def UseDashes(self, value: bool) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_HuygensMtf(IAS_):
    @property
    def Configuration(self) -> int: ...
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def ImageDelta(self) -> float: ...
    @property
    def ImageSampleSize(self) -> SampleSizes: ...
    @property
    def MaximumFrequency(self) -> float: ...
    @property
    def PupilSampleSize(self) -> SampleSizes: ...
    @property
    def Type(self) -> HuygensMtfTypes: ...
    @property
    def UseDashes(self) -> bool: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @Configuration.setter
    def Configuration(self, value: int) -> None: ...
    @ImageDelta.setter
    def ImageDelta(self, value: float) -> None: ...
    @ImageSampleSize.setter
    def ImageSampleSize(self, value: SampleSizes) -> None: ...
    @MaximumFrequency.setter
    def MaximumFrequency(self, value: float) -> None: ...
    @PupilSampleSize.setter
    def PupilSampleSize(self, value: SampleSizes) -> None: ...
    @Type.setter
    def Type(self, value: HuygensMtfTypes) -> None: ...
    @UseDashes.setter
    def UseDashes(self, value: bool) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_HuygensMtfvsField(IAS_):
    @property
    def FieldDensity(self) -> int: ...
    @property
    def Freq_1(self) -> float: ...
    @property
    def Freq_2(self) -> float: ...
    @property
    def Freq_3(self) -> float: ...
    @property
    def Freq_4(self) -> float: ...
    @property
    def Freq_5(self) -> float: ...
    @property
    def Freq_6(self) -> float: ...
    @property
    def RemoveVignetting(self) -> bool: ...
    @property
    def SampleSize(self) -> SampleSizes: ...
    @property
    def ScanType(self) -> ScanTypes: ...
    @property
    def UseDashes(self) -> bool: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @FieldDensity.setter
    def FieldDensity(self, value: int) -> None: ...
    @Freq_1.setter
    def Freq_1(self, value: float) -> None: ...
    @Freq_2.setter
    def Freq_2(self, value: float) -> None: ...
    @Freq_3.setter
    def Freq_3(self, value: float) -> None: ...
    @Freq_4.setter
    def Freq_4(self, value: float) -> None: ...
    @Freq_5.setter
    def Freq_5(self, value: float) -> None: ...
    @Freq_6.setter
    def Freq_6(self, value: float) -> None: ...
    @RemoveVignetting.setter
    def RemoveVignetting(self, value: bool) -> None: ...
    @SampleSize.setter
    def SampleSize(self, value: SampleSizes) -> None: ...
    @ScanType.setter
    def ScanType(self, value: ScanTypes) -> None: ...
    @UseDashes.setter
    def UseDashes(self, value: bool) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_HuygensSurfaceMtf(IAS_):
    @property
    def Configuration(self) -> int: ...
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def ImageDelta(self) -> float: ...
    @property
    def ImageSampleSize(self) -> SampleSizes: ...
    @property
    def PupilSampleSize(self) -> SampleSizes: ...
    @property
    def ShowAs(self) -> HuygensSurfaceMftShowAsTypes: ...
    @property
    def Type(self) -> SurfaceMtfTypes: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @Configuration.setter
    def Configuration(self, value: int) -> None: ...
    @ImageDelta.setter
    def ImageDelta(self, value: float) -> None: ...
    @ImageSampleSize.setter
    def ImageSampleSize(self, value: SampleSizes) -> None: ...
    @PupilSampleSize.setter
    def PupilSampleSize(self, value: SampleSizes) -> None: ...
    @ShowAs.setter
    def ShowAs(self, value: HuygensSurfaceMftShowAsTypes) -> None: ...
    @Type.setter
    def Type(self, value: SurfaceMtfTypes) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_HuygensThroughFocusMtf(IAS_):
    @property
    def Configuration(self) -> int: ...
    @property
    def DeltaFocus(self) -> float: ...
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def Frequency(self) -> float: ...
    @property
    def ImageDelta(self) -> float: ...
    @property
    def ImageSampleSize(self) -> SampleSizes: ...
    @property
    def NumberOfSteps(self) -> int: ...
    @property
    def PupilSampleSize(self) -> SampleSizes: ...
    @property
    def Type(self) -> HuygensMtfTypes: ...
    @property
    def UseDashes(self) -> bool: ...
    @property
    def UsePolarization(self) -> bool: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @Configuration.setter
    def Configuration(self, value: int) -> None: ...
    @DeltaFocus.setter
    def DeltaFocus(self, value: float) -> None: ...
    @Frequency.setter
    def Frequency(self, value: float) -> None: ...
    @ImageDelta.setter
    def ImageDelta(self, value: float) -> None: ...
    @ImageSampleSize.setter
    def ImageSampleSize(self, value: SampleSizes) -> None: ...
    @NumberOfSteps.setter
    def NumberOfSteps(self, value: int) -> None: ...
    @PupilSampleSize.setter
    def PupilSampleSize(self, value: SampleSizes) -> None: ...
    @Type.setter
    def Type(self, value: HuygensMtfTypes) -> None: ...
    @UseDashes.setter
    def UseDashes(self, value: bool) -> None: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...

class IAS_NSCGeometricMtf(IAS_):
    @property
    def Detector(self) -> IAS_Detector: ...
    @property
    def Filter(self) -> str: ...
    @property
    def MaximumFrequency(self) -> float: ...
    @property
    def MaximumPlotScale(self) -> float: ...
    @property
    def MinimumPlotScale(self) -> float: ...
    @property
    def RayDatabaseFilename(self) -> str: ...
    @Filter.setter
    def Filter(self, value: str) -> None: ...
    @MaximumFrequency.setter
    def MaximumFrequency(self, value: float) -> None: ...
    @MaximumPlotScale.setter
    def MaximumPlotScale(self, value: float) -> None: ...
    @MinimumPlotScale.setter
    def MinimumPlotScale(self, value: float) -> None: ...
    @RayDatabaseFilename.setter
    def RayDatabaseFilename(self, value: str) -> None: ...

class MtfDataTypes:
    Average = 0
    Tangential = 1
    Sagittal = 2

class MtfTypes:
    Modulation = 0
    Real = 1
    Imaginary = 2
    Phase = 3
    SquareWave = 4

class ShowAsTypes:
    GreyScale = 0
    GreyScaleInverted = 1
    FalseColor = 2
    FalseColorInverted = 3

class SurfaceMtfTypes:
    MTF_Incoherent = 0
    Real_Incoherent = 1
    Imaginary_Incoherent = 2
    MTF_Coherent = 3
    Real_Coherent = 4
    Imaginary_Coherent = 5



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

__all__ = ("SagittalAberrationComponent", "TangentialAberrationComponent")

class SagittalAberrationComponent(Enum):
    Aberration_X = 0
    Aberration_Y = 1

class TangentialAberrationComponent(Enum):
    Aberration_Y = 0
    Aberration_X = 1



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

from . import LDE, MCE, MFE, NCE, TDE

__all__ = (
    "LDE",
    "MCE",
    "MFE",
    "NCE",
    "TDE",
    "CellDataType",
    "EditorType",
    "ReflectTransmitCode",
    "SampleSides",
    "Samplings",
    "SolveStatus",
    "SolveType",
)

class CellDataType(Enum):
    Integer = 0
    Double = 1
    String = 2

class EditorType(Enum):
    LDE = 0
    NCE = 1
    MFE = 2
    TDE = 3
    MCE = 4

class ReflectTransmitCode(Enum):
    Success = 0
    NoReflectDataInFile = 1
    NoTransmitDataInFile = 2

class SampleSides(Enum):
    Front = 0
    Back = 1

class Samplings(Enum):
    FiveDegrees = 0
    TwoDegrees = 1
    OneDegree = 2

class SolveStatus(Enum):
    Success = 0
    InvalidSolveType = 1
    InvalidRow = 2
    InvalidColumn = 3
    PostSurfaceStopOnly = 4
    InvalidMacro = 5
    Failed = 10000

class SolveType(Enum):
    None_ = 0
    Fixed = 1
    Variable = 2
    SurfacePickup = 3
    ZPLMacro = 4
    MarginalRayAngle = 5
    MarginalRayHeight = 6
    ChiefRayAngle = 7
    MarginalRayNormal = 8
    ChiefRayNormal = 9
    Aplanatic = 10
    ElementPower = 11
    CocentricSurface = 12
    ConcentricSurface = 12
    CocentricRadius = 13
    ConcentricRadius = 13
    FNumber = 14
    ChiefRayHeight = 15
    EdgeThickness = 16
    OpticalPathDifference = 17
    Position = 18
    Compensator = 19
    CenterOfCurvature = 20
    PupilPosition = 21
    MaterialSubstitute = 22
    MaterialOffset = 23
    MaterialModel = 24
    Automatic = 25
    Maximum = 26
    PickupChiefRay = 27
    ObjectPickup = 28
    ConfigPickup = 29
    ThermalPickup = 30
    MarginPercent = 31
    CA_fill = 32
    DIA_fill = 33
    DuplicateSag = 34
    InvertSag = 35
    FieldPickup = 10000



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

__all__ = (
    "CoatingStatusType",
    "ConversionOrder",
    "CoordinateConversionResult",
    "CoordinateReturnType",
    "InterpolationMethod",
    "NodesDataType",
    "PilotRadiusMode",
    "PointCloudFileFormat",
    "PupilApodizationType",
    "QTypes",
    "STARDeformationOption",
    "SubstrateType",
    "SurfaceApertureTypes",
    "SurfaceColumn",
    "SurfaceEdgeDraw",
    "SurfaceScatteringTypes",
    "SurfaceType",
    "TiltDecenterOrderType",
    "TiltDecenterPickupType",
    "TiltType",
    "XYSampling",
)

class CoatingStatusType(Enum):
    Fixed = 0
    Variable = 1
    Pickup = 2

class ConversionOrder(Enum):
    Forward = 0
    Reverse = 1

class CoordinateConversionResult(Enum):
    Success = 0
    Error_MultiConfig = -5
    Error_TiltDecenter = -4
    Error_IgnoredSurface = -3
    Error_CoordianteBreak = -2
    Error_InvalidRange = -1

class CoordinateReturnType(Enum):
    None_ = 0
    OrientationOnly = 1
    OrientationXY = 2
    OrientationXYZ = 3

class InterpolationMethod(Enum):
    BicubicSpline = 0
    Linear = 1

class NodesDataType(Enum):
    SurfaceDeformationNoRBM = 1
    RefractiveIndex = 2
    TemperatureAndRefractiveIndex = 3
    SurfaceDeformation = 4

class PilotRadiusMode(Enum):
    BestFit = 0
    Shorter = 1
    Longer = 2
    X = 3
    Y = 4
    Plane = 5
    User = 6

class PointCloudFileFormat(Enum):
    ASCII = 0
    Binary = 1
    CompressedBinary = 2

class PupilApodizationType(Enum):
    None_ = 0
    Gaussian = 1
    Tangential = 2

class QTypes(Enum):
    Qbfs = 0
    Qcon = 1

class STARDeformationOption(Enum):
    DeformationWithRBMs = 0
    DeformationWithoutRBMs = 1
    OnlyRBMs = 2
    NoDeformation = 3

class SubstrateType(Enum):
    None_ = 0
    Flat = 1
    Curved = 2

class SurfaceApertureTypes(Enum):
    None_ = 0
    CircularAperture = 1
    CircularObscuration = 2
    Spider = 3
    RectangularAperture = 4
    RectangularObscuration = 5
    EllipticalAperture = 6
    EllipticalObscuration = 7
    UserAperture = 8
    UserObscuration = 9
    FloatingAperture = 10

class SurfaceColumn(Enum):
    Comment = 1
    Radius = 2
    Thickness = 3
    Material = 4
    Coating = 5
    SemiDiameter = 6
    ChipZone = 7
    MechanicalSemiDiameter = 8
    Conic = 9
    TCE = 10
    Par0 = 11
    Par1 = 12
    Par2 = 13
    Par3 = 14
    Par4 = 15
    Par5 = 16
    Par6 = 17
    Par7 = 18
    Par8 = 19
    Par9 = 20
    Par10 = 21
    Par11 = 22
    Par12 = 23
    Par13 = 24
    Par14 = 25
    Par15 = 26
    Par16 = 27
    Par17 = 28
    Par18 = 29
    Par19 = 30
    Par20 = 31
    Par21 = 32
    Par22 = 33
    Par23 = 34
    Par24 = 35
    Par25 = 36
    Par26 = 37
    Par27 = 38
    Par28 = 39
    Par29 = 40
    Par30 = 41
    Par31 = 42
    Par32 = 43
    Par33 = 44
    Par34 = 45
    Par35 = 46
    Par36 = 47
    Par37 = 48
    Par38 = 49
    Par39 = 50
    Par40 = 51
    Par41 = 52
    Par42 = 53
    Par43 = 54
    Par44 = 55
    Par45 = 56
    Par46 = 57
    Par47 = 58
    Par48 = 59
    Par49 = 60
    Par50 = 61
    Par51 = 62
    Par52 = 63
    Par53 = 64
    Par54 = 65
    Par55 = 66
    Par56 = 67
    Par57 = 68
    Par58 = 69
    Par59 = 70
    Par60 = 71
    Par61 = 72
    Par62 = 73
    Par63 = 74
    Par64 = 75
    Par65 = 76
    Par66 = 77
    Par67 = 78
    Par68 = 79
    Par69 = 80
    Par70 = 81
    Par71 = 82
    Par72 = 83
    Par73 = 84
    Par74 = 85
    Par75 = 86
    Par76 = 87
    Par77 = 88
    Par78 = 89
    Par79 = 90
    Par80 = 91
    Par81 = 92
    Par82 = 93
    Par83 = 94
    Par84 = 95
    Par85 = 96
    Par86 = 97
    Par87 = 98
    Par88 = 99
    Par89 = 100
    Par90 = 101
    Par91 = 102
    Par92 = 103
    Par93 = 104
    Par94 = 105
    Par95 = 106
    Par96 = 107
    Par97 = 108
    Par98 = 109
    Par99 = 110
    Par100 = 111
    Par101 = 112
    Par102 = 113
    Par103 = 114
    Par104 = 115
    Par105 = 116
    Par106 = 117
    Par107 = 118
    Par108 = 119
    Par109 = 120
    Par110 = 121
    Par111 = 122
    Par112 = 123
    Par113 = 124
    Par114 = 125
    Par115 = 126
    Par116 = 127
    Par117 = 128
    Par118 = 129
    Par119 = 130
    Par120 = 131
    Par121 = 132
    Par122 = 133
    Par123 = 134
    Par124 = 135
    Par125 = 136
    Par126 = 137
    Par127 = 138
    Par128 = 139
    Par129 = 140
    Par130 = 141
    Par131 = 142
    Par132 = 143
    Par133 = 144
    Par134 = 145
    Par135 = 146
    Par136 = 147
    Par137 = 148
    Par138 = 149
    Par139 = 150
    Par140 = 151
    Par141 = 152
    Par142 = 153
    Par143 = 154
    Par144 = 155
    Par145 = 156
    Par146 = 157
    Par147 = 158
    Par148 = 159
    Par149 = 160
    Par150 = 161
    Par151 = 162
    Par152 = 163
    Par153 = 164
    Par154 = 165
    Par155 = 166
    Par156 = 167
    Par157 = 168
    Par158 = 169
    Par159 = 170
    Par160 = 171
    Par161 = 172
    Par162 = 173
    Par163 = 174
    Par164 = 175
    Par165 = 176
    Par166 = 177
    Par167 = 178
    Par168 = 179
    Par169 = 180
    Par170 = 181
    Par171 = 182
    Par172 = 183
    Par173 = 184
    Par174 = 185
    Par175 = 186
    Par176 = 187
    Par177 = 188
    Par178 = 189
    Par179 = 190
    Par180 = 191
    Par181 = 192
    Par182 = 193
    Par183 = 194
    Par184 = 195
    Par185 = 196
    Par186 = 197
    Par187 = 198
    Par188 = 199
    Par189 = 200
    Par190 = 201
    Par191 = 202
    Par192 = 203
    Par193 = 204
    Par194 = 205
    Par195 = 206
    Par196 = 207
    Par197 = 208
    Par198 = 209
    Par199 = 210
    Par200 = 211
    Par201 = 212
    Par202 = 213
    Par203 = 214
    Par204 = 215
    Par205 = 216
    Par206 = 217
    Par207 = 218
    Par208 = 219
    Par209 = 220
    Par210 = 221
    Par211 = 222
    Par212 = 223
    Par213 = 224
    Par214 = 225
    Par215 = 226
    Par216 = 227
    Par217 = 228
    Par218 = 229
    Par219 = 230
    Par220 = 231
    Par221 = 232
    Par222 = 233
    Par223 = 234
    Par224 = 235
    Par225 = 236
    Par226 = 237
    Par227 = 238
    Par228 = 239
    Par229 = 240
    Par230 = 241
    Par231 = 242
    Par232 = 243
    Par233 = 244
    Par234 = 245
    Par235 = 246
    Par236 = 247
    Par237 = 248
    Par238 = 249
    Par239 = 250
    Par240 = 251
    Par241 = 252
    Par242 = 253
    Par243 = 254
    Par244 = 255
    Par245 = 256
    Par246 = 257
    Par247 = 258
    Par248 = 259
    Par249 = 260
    Par250 = 261
    Par251 = 262
    Par252 = 263
    Par253 = 264
    Par254 = 265

class SurfaceEdgeDraw(Enum):
    Squared = 0
    Tapered = 1
    Flat = 2

class SurfaceScatteringTypes(Enum):
    None_ = 0
    Lambertian = 1
    Gaussian = 2
    ABg = 3
    ABgFile = 4
    BSDF = 5
    User = 6
    ISScatterCatalog = 7

class SurfaceType(Enum):
    ABCD = 0
    AlternateEven = 1
    AlternateOdd = 2
    AnnularZernikeSag = 3
    Atmospheric = 4
    Biconic = 5
    BiconicZernike = 6
    Binary1 = 7
    Binary2 = 8
    Binary3 = 9
    Binary4 = 10
    BirefringentIn = 11
    BirefringentOut = 12
    BlackBoxLens = 13
    ChebyShv = 14
    Conjugate = 15
    CoordinateBreak = 16
    CubicSpline = 17
    CylinderFrensel = 18
    CylinderFresnel = 18
    Data = 19
    DiffractionGrating = 20
    EllipticalGrating1 = 21
    EllipticalGrating2 = 22
    EvenAspheric = 23
    ExtendedToroidalGrating = 24
    ExtendedAsphere = 25
    ExtendedCubicSpline = 26
    ExtendedFresnel = 27
    ExtendedOddAsphere = 28
    ExtendedPolynomial = 29
    Fresnel = 30
    GeneralizedFresnel = 31
    Gradient1 = 32
    Gradient2 = 33
    Gradient3 = 34
    Gradient4 = 35
    Gradient5 = 36
    Gradient6 = 37
    Gradient7 = 38
    Gradient9 = 39
    Gradient10 = 40
    Gradient12 = 41
    Gradium = 42
    GridGradient = 43
    GridPhase = 44
    GridSag = 45
    Hologram1 = 46
    Hologram2 = 47
    Irregular = 48
    JonesMatrix = 49
    NonSequential = 50
    OddAsphere = 51
    OddCosine = 52
    OffAxisConicFreeform = 53
    OpticallyFabricatedHologram = 54
    Paraxial = 55
    ParaxialXY = 56
    Periodic = 57
    Polynomial = 58
    QTypeAsphere = 59
    QTypeFreeform = 60
    RadialGrating = 61
    RadialNurbs = 62
    RetroReflect = 63
    SlideSurface = 64
    Standard = 65
    Superconic = 66
    Tilted = 67
    Toroidal = 68
    ToroidalGrat = 69
    ToroidalHologram = 70
    ToroidalNurbs = 71
    UserDefined = 72
    VariableLineSpaceGrating = 73
    ZernikeAnnularPhase = 74
    ZernikeFringePhase = 75
    ZernikeFringeSag = 76
    ZernikeStandardPhase = 77
    ZernikeStandardSag = 78
    ZonePlate = 79
    Freeform = 80

class TiltDecenterOrderType(Enum):
    Decenter_Tilt = 0
    Tilt_Decenter = 1

class TiltDecenterPickupType(Enum):
    Explicit = 0
    PickupSurface = 1
    ReverseSurface = 2

class TiltType(Enum):
    XTilt = 0
    YTilt = 1

class XYSampling(Enum):
    S32 = 0
    S64 = 1
    S128 = 2
    S256 = 3
    S512 = 4
    S1024 = 5
    S2048 = 6
    S4096 = 7
    S8192 = 8
    S16384 = 9



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

__all__ = "PathAnalysisSortType"

class PathAnalysisSortType(Enum):
    TotalEndingFlux = 0
    NumberOfObjectsStruck = 1
    NumberOfBranches = 2
    LastObjectStruck = 3
    NumberOfUniqueObjectsStruct = 4



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from typing import Iterable, overload

from zospy.api._ZOSAPI.Editors.LDE import ILDERow
from zospy.api._ZOSAPI.Tools.General import IDataDictionary

__all__ = (
    "DrawingCementData",
    "DrawingCementData",
    "DrawingComponentData",
    "DrawingComponentData",
    "DrawingMaterialData",
    "DrawingMaterialData",
    "DrawingSurfaceData",
    "DrawingSurfaceData",
    "LMXF_GeneralSettings",
    "LMXF_GeneralSettings",
    "LMXF_NscConfigData",
    "LMXF_NscConfigData",
    "LMXF_NscData",
    "LMXF_NscData",
    "LMXF_NscSourceData",
    "LMXF_NscSourceData",
    "LMXF_OpticalAxisSurfaceData",
    "LMXF_OpticalAxisSurfaceData",
    "LMXF_Platforms",
    "LMXF_SeqConfigData",
    "LMXF_SeqConfigData",
    "LMXF_SeqData",
    "LMXF_SeqData",
    "LMXF_Wrapper",
    "LMXF_Wrapper",
)

class DrawingCementData:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, cement: DrawingCementData): ...
    @property
    def CementType(self) -> str: ...
    @property
    def CementWedge(self) -> Nullable: ...
    @property
    def LocalizedImpTol(self) -> Nullable: ...
    @property
    def RotationalInvariant(self) -> Nullable: ...
    @property
    def SurfaceImpTol(self) -> Nullable: ...
    @property
    def TransmitWavefront(self) -> Nullable: ...
    @CementType.setter
    def CementType(self, value: str) -> None: ...
    @CementWedge.setter
    def CementWedge(self, value: Nullable) -> None: ...
    @LocalizedImpTol.setter
    def LocalizedImpTol(self, value: Nullable) -> None: ...
    @RotationalInvariant.setter
    def RotationalInvariant(self, value: Nullable) -> None: ...
    @SurfaceImpTol.setter
    def SurfaceImpTol(self, value: Nullable) -> None: ...
    @TransmitWavefront.setter
    def TransmitWavefront(self, value: Nullable) -> None: ...

class DrawingComponentData:
    @overload
    def __init__(self, isSequentialMode: bool): ...
    @overload
    def __init__(self, component: DrawingComponentData, isSequentialMode: bool): ...
    @property
    def BackSurface(self) -> DrawingSurfaceData: ...
    @property
    def BackSurfaceNumber(self) -> int: ...
    @property
    def CementData(self) -> DrawingCementData: ...
    @property
    def ElementType(self) -> int: ...
    @property
    def FrontSurface(self) -> DrawingSurfaceData: ...
    @property
    def FrontSurfaceNumber(self) -> int: ...
    @property
    def IsDrawingInputsSupportedForBackSurface(self) -> bool: ...
    @property
    def IsDrawingInputsSupportedForFrontSurface(self) -> bool: ...
    @property
    def IsLeadElement(self) -> bool: ...
    @property
    def Material(self) -> DrawingMaterialData: ...
    @property
    def ObjectNumber(self) -> int: ...
    @BackSurface.setter
    def BackSurface(self, value: DrawingSurfaceData) -> None: ...
    @BackSurfaceNumber.setter
    def BackSurfaceNumber(self, value: int) -> None: ...
    @CementData.setter
    def CementData(self, value: DrawingCementData) -> None: ...
    @ElementType.setter
    def ElementType(self, value: int) -> None: ...
    @FrontSurface.setter
    def FrontSurface(self, value: DrawingSurfaceData) -> None: ...
    @FrontSurfaceNumber.setter
    def FrontSurfaceNumber(self, value: int) -> None: ...
    @IsLeadElement.setter
    def IsLeadElement(self, value: bool) -> None: ...
    @Material.setter
    def Material(self, value: DrawingMaterialData) -> None: ...
    @ObjectNumber.setter
    def ObjectNumber(self, value: int) -> None: ...

class DrawingMaterialData:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, material: DrawingMaterialData): ...
    @property
    def AbbeNumber(self) -> Nullable: ...
    @property
    def AbbeTolerance(self) -> Nullable: ...
    @property
    def BubblesInclusions(self) -> str: ...
    @property
    def Dig(self) -> Nullable: ...
    @property
    def IndexOfRefraction(self) -> Nullable: ...
    @property
    def IndexTolerance(self) -> Nullable: ...
    @property
    def InhomogeneityStriae(self) -> str: ...
    @property
    def Material(self) -> str: ...
    @property
    def ReferenceWL(self) -> Nullable: ...
    @property
    def Scratch(self) -> Nullable: ...
    @property
    def StressBirefringence(self) -> Nullable: ...
    @property
    def TestWavelengthNM(self) -> Nullable: ...
    @property
    def Thickness(self) -> Nullable: ...
    @property
    def ThicknessTolerance(self) -> Nullable: ...
    @property
    def TransmittedWavefront(self) -> str: ...
    @property
    def WavefrontDeformTol(self) -> str: ...
    @AbbeNumber.setter
    def AbbeNumber(self, value: Nullable) -> None: ...
    @AbbeTolerance.setter
    def AbbeTolerance(self, value: Nullable) -> None: ...
    @BubblesInclusions.setter
    def BubblesInclusions(self, value: str) -> None: ...
    @Dig.setter
    def Dig(self, value: Nullable) -> None: ...
    @IndexOfRefraction.setter
    def IndexOfRefraction(self, value: Nullable) -> None: ...
    @IndexTolerance.setter
    def IndexTolerance(self, value: Nullable) -> None: ...
    @InhomogeneityStriae.setter
    def InhomogeneityStriae(self, value: str) -> None: ...
    @Material.setter
    def Material(self, value: str) -> None: ...
    @ReferenceWL.setter
    def ReferenceWL(self, value: Nullable) -> None: ...
    @Scratch.setter
    def Scratch(self, value: Nullable) -> None: ...
    @StressBirefringence.setter
    def StressBirefringence(self, value: Nullable) -> None: ...
    @TestWavelengthNM.setter
    def TestWavelengthNM(self, value: Nullable) -> None: ...
    @Thickness.setter
    def Thickness(self, value: Nullable) -> None: ...
    @ThicknessTolerance.setter
    def ThicknessTolerance(self, value: Nullable) -> None: ...
    @TransmittedWavefront.setter
    def TransmittedWavefront(self, value: str) -> None: ...
    @WavefrontDeformTol.setter
    def WavefrontDeformTol(self, value: str) -> None: ...

class DrawingSurfaceData:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, data: DrawingSurfaceData): ...
    def CreateRange(rMin: float, rMax: float, numR: int) -> Iterable[float]: ...
    @property
    def AsphereCoefficients(self) -> list[float]: ...
    @property
    def CentrationDecentration(self) -> Nullable: ...
    @property
    def ClearAperture(self) -> Nullable: ...
    @property
    def ClearApertureTolerance(self) -> Nullable: ...
    @property
    def Coating(self) -> str: ...
    @property
    def Conic(self) -> Nullable: ...
    @property
    def FTWMax(self) -> Nullable: ...
    @property
    def FTWMin(self) -> Nullable: ...
    @property
    def FTWPct(self) -> Nullable: ...
    @property
    def ImperfectionTolerance(self) -> str: ...
    @property
    def IrregularityFormError(self) -> str: ...
    @property
    def IsAspheric(self) -> bool: ...
    @property
    def IsDrawingInputsSupported(self) -> bool: ...
    @property
    def LaserPowerDamage(self) -> str: ...
    @property
    def PaintNotes(self) -> str: ...
    @property
    def PowerIrr(self) -> str: ...
    @property
    def ProtectionChamfer(self) -> Nullable: ...
    @property
    def Radius(self) -> Nullable: ...
    @property
    def RadiusTolerance(self) -> Nullable: ...
    @property
    def ReferenceWavelengthNM(self) -> Nullable: ...
    @property
    def SagData(self) -> list[float]: ...
    @property
    def SagDataSemiDiameter(self) -> Nullable: ...
    @property
    def SagTable(self) -> list[list[float]]: ...
    @property
    def SagTableHeight(self) -> Nullable: ...
    @property
    def SagTableWidth(self) -> Nullable: ...
    @property
    def SlopeError(self) -> Nullable: ...
    @property
    def SurfaceName(self) -> str: ...
    @property
    def SurfaceTexture(self) -> str: ...
    @property
    def SurfaceType(self) -> str: ...
    @property
    def TiltWedge(self) -> Nullable: ...
    @property
    def VertexRadius(self) -> Nullable: ...
    @property
    def VertexRadiusTolerance(self) -> Nullable: ...
    def GetAsphericSag(self, radii: Iterable[float]) -> List: ...
    def GetAsphericSagAtRadius(self, r: float) -> float: ...
    def GetInvariantSurfaceTypeName(surface: ILDERow) -> str: ...
    @AsphereCoefficients.setter
    def AsphereCoefficients(self, value: list[float]) -> None: ...
    @CentrationDecentration.setter
    def CentrationDecentration(self, value: Nullable) -> None: ...
    @ClearAperture.setter
    def ClearAperture(self, value: Nullable) -> None: ...
    @ClearApertureTolerance.setter
    def ClearApertureTolerance(self, value: Nullable) -> None: ...
    @Coating.setter
    def Coating(self, value: str) -> None: ...
    @Conic.setter
    def Conic(self, value: Nullable) -> None: ...
    @FTWMax.setter
    def FTWMax(self, value: Nullable) -> None: ...
    @FTWMin.setter
    def FTWMin(self, value: Nullable) -> None: ...
    @FTWPct.setter
    def FTWPct(self, value: Nullable) -> None: ...
    @ImperfectionTolerance.setter
    def ImperfectionTolerance(self, value: str) -> None: ...
    @IrregularityFormError.setter
    def IrregularityFormError(self, value: str) -> None: ...
    @LaserPowerDamage.setter
    def LaserPowerDamage(self, value: str) -> None: ...
    @PaintNotes.setter
    def PaintNotes(self, value: str) -> None: ...
    @PowerIrr.setter
    def PowerIrr(self, value: str) -> None: ...
    @ProtectionChamfer.setter
    def ProtectionChamfer(self, value: Nullable) -> None: ...
    @Radius.setter
    def Radius(self, value: Nullable) -> None: ...
    @RadiusTolerance.setter
    def RadiusTolerance(self, value: Nullable) -> None: ...
    @ReferenceWavelengthNM.setter
    def ReferenceWavelengthNM(self, value: Nullable) -> None: ...
    @SagData.setter
    def SagData(self, value: list[float]) -> None: ...
    @SagDataSemiDiameter.setter
    def SagDataSemiDiameter(self, value: Nullable) -> None: ...
    @SagTable.setter
    def SagTable(self, value: list[list[float]]) -> None: ...
    @SagTableHeight.setter
    def SagTableHeight(self, value: Nullable) -> None: ...
    @SagTableWidth.setter
    def SagTableWidth(self, value: Nullable) -> None: ...
    @SlopeError.setter
    def SlopeError(self, value: Nullable) -> None: ...
    @SurfaceName.setter
    def SurfaceName(self, value: str) -> None: ...
    @SurfaceTexture.setter
    def SurfaceTexture(self, value: str) -> None: ...
    @SurfaceType.setter
    def SurfaceType(self, value: str) -> None: ...
    @TiltWedge.setter
    def TiltWedge(self, value: Nullable) -> None: ...
    @VertexRadius.setter
    def VertexRadius(self, value: Nullable) -> None: ...
    @VertexRadiusTolerance.setter
    def VertexRadiusTolerance(self, value: Nullable) -> None: ...

class LMXF_GeneralSettings:
    def __init__(self): ...
    @property
    def ConvertStopToNSCAperture(self) -> bool: ...
    @property
    def IncludeScatter(self) -> bool: ...
    @property
    def IncludeSplitting(self) -> bool: ...
    @property
    def IsEditable(self) -> bool: ...
    @property
    def Notes(self) -> str: ...
    @property
    def Precision(self) -> int: ...
    @property
    def StopMechanicalHalfWidth(self) -> float: ...
    @property
    def TargetPlatform(self) -> LMXF_Platforms: ...
    @ConvertStopToNSCAperture.setter
    def ConvertStopToNSCAperture(self, value: bool) -> None: ...
    @IncludeScatter.setter
    def IncludeScatter(self, value: bool) -> None: ...
    @IncludeSplitting.setter
    def IncludeSplitting(self, value: bool) -> None: ...
    @IsEditable.setter
    def IsEditable(self, value: bool) -> None: ...
    @Notes.setter
    def Notes(self, value: str) -> None: ...
    @Precision.setter
    def Precision(self, value: int) -> None: ...
    @StopMechanicalHalfWidth.setter
    def StopMechanicalHalfWidth(self, value: float) -> None: ...
    @TargetPlatform.setter
    def TargetPlatform(self, value: LMXF_Platforms) -> None: ...

class LMXF_NscConfigData:
    def __init__(self): ...
    @property
    def ComponentObjects(self) -> list[int]: ...
    @property
    def ComponentPositions(self) -> list[list[float]]: ...
    @property
    def DrawingData(self) -> list[DrawingComponentData]: ...
    @property
    def NscSourceData(self) -> LMXF_NscSourceData: ...
    @property
    def OriginalConfigNumber(self) -> int: ...
    @ComponentObjects.setter
    def ComponentObjects(self, value: list[int]) -> None: ...
    @ComponentPositions.setter
    def ComponentPositions(self, value: list[list[float]]) -> None: ...
    @DrawingData.setter
    def DrawingData(self, value: list[DrawingComponentData]) -> None: ...
    @NscSourceData.setter
    def NscSourceData(self, value: LMXF_NscSourceData) -> None: ...
    @OriginalConfigNumber.setter
    def OriginalConfigNumber(self, value: int) -> None: ...

class LMXF_NscData:
    def __init__(self): ...
    @property
    def ConfigData(self) -> list[LMXF_NscConfigData]: ...
    @property
    def IsNscSource(self) -> bool: ...
    @property
    def NSCFileKey(self) -> str: ...
    @property
    def NSCFileName(self) -> str: ...
    @property
    def StopObjectFace(self) -> int: ...
    @property
    def StopObjectNumber(self) -> int: ...
    @ConfigData.setter
    def ConfigData(self, value: list[LMXF_NscConfigData]) -> None: ...
    @IsNscSource.setter
    def IsNscSource(self, value: bool) -> None: ...
    @NSCFileKey.setter
    def NSCFileKey(self, value: str) -> None: ...
    @NSCFileName.setter
    def NSCFileName(self, value: str) -> None: ...
    @StopObjectFace.setter
    def StopObjectFace(self, value: int) -> None: ...
    @StopObjectNumber.setter
    def StopObjectNumber(self, value: int) -> None: ...

class LMXF_NscSourceData:
    def __init__(self): ...
    @property
    def DetectorDataKeys(self) -> list[str]: ...
    @property
    def DetectorObjects(self) -> list[int]: ...
    @property
    def IntendedPathFlux(self) -> list[float]: ...
    @property
    def IntendedPaths(self) -> list[list[int]]: ...
    @property
    def NscMtfData(self) -> list[list[float]]: ...
    @property
    def NscSpotImageKeys(self) -> list[str]: ...
    @property
    def NscSpotSizes(self) -> list[float]: ...
    @property
    def SourceObjects(self) -> list[int]: ...
    @property
    def TotalFlux(self) -> float: ...
    @DetectorDataKeys.setter
    def DetectorDataKeys(self, value: list[str]) -> None: ...
    @DetectorObjects.setter
    def DetectorObjects(self, value: list[int]) -> None: ...
    @IntendedPathFlux.setter
    def IntendedPathFlux(self, value: list[float]) -> None: ...
    @IntendedPaths.setter
    def IntendedPaths(self, value: list[list[int]]) -> None: ...
    @NscMtfData.setter
    def NscMtfData(self, value: list[list[float]]) -> None: ...
    @NscSpotImageKeys.setter
    def NscSpotImageKeys(self, value: list[str]) -> None: ...
    @NscSpotSizes.setter
    def NscSpotSizes(self, value: list[float]) -> None: ...
    @SourceObjects.setter
    def SourceObjects(self, value: list[int]) -> None: ...
    @TotalFlux.setter
    def TotalFlux(self, value: float) -> None: ...

class LMXF_OpticalAxisSurfaceData:
    @property
    def AreCoordinatesGlobal(self) -> bool: ...
    @property
    def ErrorCode(self) -> int: ...
    @property
    def Nx(self) -> float: ...
    @property
    def Ny(self) -> float: ...
    @property
    def Nz(self) -> float: ...
    @property
    def SurfaceNumber(self) -> int: ...
    @property
    def VignetteCode(self) -> int: ...
    @property
    def X(self) -> float: ...
    @property
    def Y(self) -> float: ...
    @property
    def Z(self) -> float: ...
    @AreCoordinatesGlobal.setter
    def AreCoordinatesGlobal(self, value: bool) -> None: ...
    @ErrorCode.setter
    def ErrorCode(self, value: int) -> None: ...
    @Nx.setter
    def Nx(self, value: float) -> None: ...
    @Ny.setter
    def Ny(self, value: float) -> None: ...
    @Nz.setter
    def Nz(self, value: float) -> None: ...
    @SurfaceNumber.setter
    def SurfaceNumber(self, value: int) -> None: ...
    @VignetteCode.setter
    def VignetteCode(self, value: int) -> None: ...
    @X.setter
    def X(self, value: float) -> None: ...
    @Y.setter
    def Y(self, value: float) -> None: ...
    @Z.setter
    def Z(self, value: float) -> None: ...

class LMXF_Platforms:
    Unknown = 0
    SolidWorks = 1
    Creo = 2

class LMXF_SeqConfigData:
    def __init__(self): ...
    @property
    def ComponentPositions(self) -> list[list[float]]: ...
    @property
    def ComponentSurfaces(self) -> list[int]: ...
    @property
    def CrsFileKey(self) -> str: ...
    @property
    def DetectorDataKeys(self) -> list[str]: ...
    @property
    def DetectorObjects(self) -> list[int]: ...
    @property
    def FieldNumbers(self) -> list[int]: ...
    @property
    def FullFieldKey(self) -> str: ...
    @property
    def IntendedPathFlux(self) -> list[float]: ...
    @property
    def IntendedPaths(self) -> list[list[int]]: ...
    @property
    def NscMtfData(self) -> list[list[float]]: ...
    @property
    def NscSpotImageKeys(self) -> list[str]: ...
    @property
    def NscSpotSizes(self) -> list[float]: ...
    @property
    def OpticalAxisData(self) -> list[LMXF_OpticalAxisSurfaceData]: ...
    @property
    def OriginalConfigNumber(self) -> int: ...
    @property
    def SeqMtfData(self) -> list[list[float]]: ...
    @property
    def SeqSpotImageKeys(self) -> list[str]: ...
    @property
    def SeqSpotSizes(self) -> list[float]: ...
    @property
    def SourceObjects(self) -> list[int]: ...
    @property
    def TotalFlux(self) -> float: ...
    @property
    def ZrdFileKey(self) -> str: ...
    @ComponentPositions.setter
    def ComponentPositions(self, value: list[list[float]]) -> None: ...
    @ComponentSurfaces.setter
    def ComponentSurfaces(self, value: list[int]) -> None: ...
    @CrsFileKey.setter
    def CrsFileKey(self, value: str) -> None: ...
    @DetectorDataKeys.setter
    def DetectorDataKeys(self, value: list[str]) -> None: ...
    @DetectorObjects.setter
    def DetectorObjects(self, value: list[int]) -> None: ...
    @FieldNumbers.setter
    def FieldNumbers(self, value: list[int]) -> None: ...
    @FullFieldKey.setter
    def FullFieldKey(self, value: str) -> None: ...
    @IntendedPathFlux.setter
    def IntendedPathFlux(self, value: list[float]) -> None: ...
    @IntendedPaths.setter
    def IntendedPaths(self, value: list[list[int]]) -> None: ...
    @NscMtfData.setter
    def NscMtfData(self, value: list[list[float]]) -> None: ...
    @NscSpotImageKeys.setter
    def NscSpotImageKeys(self, value: list[str]) -> None: ...
    @NscSpotSizes.setter
    def NscSpotSizes(self, value: list[float]) -> None: ...
    @OpticalAxisData.setter
    def OpticalAxisData(self, value: list[LMXF_OpticalAxisSurfaceData]) -> None: ...
    @OriginalConfigNumber.setter
    def OriginalConfigNumber(self, value: int) -> None: ...
    @SeqMtfData.setter
    def SeqMtfData(self, value: list[list[float]]) -> None: ...
    @SeqSpotImageKeys.setter
    def SeqSpotImageKeys(self, value: list[str]) -> None: ...
    @SeqSpotSizes.setter
    def SeqSpotSizes(self, value: list[float]) -> None: ...
    @SourceObjects.setter
    def SourceObjects(self, value: list[int]) -> None: ...
    @TotalFlux.setter
    def TotalFlux(self, value: float) -> None: ...
    @ZrdFileKey.setter
    def ZrdFileKey(self, value: str) -> None: ...

class LMXF_SeqData:
    def __init__(self): ...
    @property
    def ConfigData(self) -> list[LMXF_SeqConfigData]: ...
    @property
    def CriticalRayAngleTolerance(self) -> float: ...
    @property
    def CriticalRayPositionTolerance(self) -> float: ...
    @property
    def DeltaBeamClipPct(self) -> float: ...
    @property
    def DeltaImgContPct(self) -> float: ...
    @property
    def DeltaSpotPct(self) -> float: ...
    @property
    def DeltaSpotUM(self) -> float: ...
    @property
    def DeltaSpotUsePct(self) -> bool: ...
    @property
    def IsCopiedData(self) -> bool: ...
    @property
    def MaxMtfFreq(self) -> float: ...
    @property
    def NumberOfAnalysisRays(self) -> int: ...
    @property
    def SEQFileKey(self) -> str: ...
    @property
    def SEQFileName(self) -> str: ...
    @ConfigData.setter
    def ConfigData(self, value: list[LMXF_SeqConfigData]) -> None: ...
    @CriticalRayAngleTolerance.setter
    def CriticalRayAngleTolerance(self, value: float) -> None: ...
    @CriticalRayPositionTolerance.setter
    def CriticalRayPositionTolerance(self, value: float) -> None: ...
    @DeltaBeamClipPct.setter
    def DeltaBeamClipPct(self, value: float) -> None: ...
    @DeltaImgContPct.setter
    def DeltaImgContPct(self, value: float) -> None: ...
    @DeltaSpotPct.setter
    def DeltaSpotPct(self, value: float) -> None: ...
    @DeltaSpotUM.setter
    def DeltaSpotUM(self, value: float) -> None: ...
    @DeltaSpotUsePct.setter
    def DeltaSpotUsePct(self, value: bool) -> None: ...
    @IsCopiedData.setter
    def IsCopiedData(self, value: bool) -> None: ...
    @MaxMtfFreq.setter
    def MaxMtfFreq(self, value: float) -> None: ...
    @NumberOfAnalysisRays.setter
    def NumberOfAnalysisRays(self, value: int) -> None: ...
    @SEQFileKey.setter
    def SEQFileKey(self, value: str) -> None: ...
    @SEQFileName.setter
    def SEQFileName(self, value: str) -> None: ...

class LMXF_Wrapper:
    def __init__(self, dataFile: IDataDictionary): ...
    def GetGeneralSettings(self) -> LMXF_GeneralSettings: ...
    def GetNscData(self) -> LMXF_NscData: ...
    def GetSeqData(self) -> LMXF_SeqData: ...
    def SaveConfigCRS(self, cfgData: LMXF_SeqConfigData, fileName: str) -> None: ...
    def SaveConfigDetector(self, cfgData: LMXF_SeqConfigData, fieldIdx: int, fileName: str) -> None: ...
    def SaveConfigFullFieldDetector(self, cfgData: LMXF_SeqConfigData, fileName: str) -> None: ...
    def SaveConfigZRD(self, cfgData: LMXF_SeqConfigData, fileName: str) -> None: ...
    def SaveNscZAR(self, nscData: LMXF_NscData, fileName: str) -> bool: ...
    def SaveSeqZAR(self, seqData: LMXF_SeqData, fileName: str) -> None: ...
    def SaveSpotFile(self, cfgData: LMXF_SeqConfigData, seqFile: bool, fieldIdx: int, fileName: str) -> None: ...



from __future__ import annotations

import functools
from collections.abc import Callable

import zospy.api.config as _config


def rsetattr(obj, attr, val):
    """Wrapper for the setattr() function that handles nested strings.

    Parameters
    ----------
    obj
        The object from which the attribute is set
    attr
        The name of the attribute. Can be nested, e.g. 'aa.bb.cc'
    val
        The value to which the attribute is set

    Returns
    -------
        None
    """
    pre, _, post = attr.rpartition(".")
    return setattr(rgetattr(obj, pre) if pre else obj, post, val)


def rgetattr(obj, attr, *args):
    """Wrapper for the getattr() function that handles nested strings.

    Parameters
    ----------
    obj
        The object from which the attribute is obtained
    attr
        The name of the attribute. Can be nested, e.g. 'aa.bb.cc'
    *args
        [default,] The default return if the attribute is not found. If not supplied, AttributeError can be
        raised

    Returns
    -------
    attribute
        The attribute or the default return when not the attribute is not found

    Raises
    ------
    AttributeError
        When the attribute does not exist and no default is supplied in the *args
    """

    def _getattr(subobj, subattr, *subargs):
        return getattr(subobj, subattr, *subargs)

    return functools.reduce(lambda x, y: _getattr(x, y, *args), [obj] + attr.split("."))


def _delocalize(
    string: str,
    decimal_point: str = ...,
    thousands_separator: str | None = ...,
) -> str:
    """Delocalize a string as a normalized number.

    By default, the locale settings stored in zospy.api.config are used.

    Parameters
    ----------
    string: str
        The string that is to be converted.
    decimal_point: str
        The decimal point separator used in the string. Defaults to zospy.api.config.DECIMAL_POINT.
    thousands_separator: str | None
        The thousands separator used in the string. Defaults to zospy.api.config.THOUSANDS_SEPARATOR.

    Returns
    -------
    str
        The delocalized string.
    """
    # Get rid of the thousands grouping
    thousands_separator = _config.THOUSANDS_SEPARATOR if thousands_separator is ... else thousands_separator
    decimal_point = _config.DECIMAL_POINT if decimal_point is ... else decimal_point

    if thousands_separator:
        string = string.replace(thousands_separator, "")

    # Replace the decimal point with a dot
    if decimal_point:
        string = string.replace(decimal_point, ".")

    return string


def atox(
    string: str,
    dtype: Callable[[str], int | float] = float,
    decimal_point: str = ...,
    thousands_separator: str | None = ...,
) -> int | float:
    """Parses a string as a number format.

    By default, the locale settings stored in zospy.api.config are used to delocalize the string.

    Parameters
    ----------
    string: str
        The string that is to be converted.
    dtype: Callable[[str], int | float]
        The function used to convert the delocalized string into a number.
    decimal_point: str
        The decimal point separator used in the string. Defaults to zospy.api.config.DECIMAL_POINT.
    thousands_separator: str | None
        The thousands separator used in the string. Defaults to zospy.api.config.THOUSANDS_SEPARATOR.

    Returns
    -------
    str
        The delocalized string.
    """
    return dtype(_delocalize(string, decimal_point=decimal_point, thousands_separator=thousands_separator))


def xtoa(
    number: float | int,
    decimal_point: str = ...,
    thousands_separator: str | None = ...,
) -> str:
    """Localizes a number back to a string suing the locale settings.

    By default, the locale settings stored in zospy.api.config are used.

    Parameters
    ----------
    number: int | float
        The number that is to be converted.
    decimal_point: str
        The decimal point separator used in the string. Defaults to zospy.api.config.DECIMAL_POINT.
    thousands_separator: str | None
        The thousands separator used in the string. Defaults to zospy.api.config.THOUSANDS_SEPARATOR.

    Returns
    -------
    str
        The localized string representation of the number.
    """
    thousands_separator = _config.THOUSANDS_SEPARATOR if thousands_separator is ... else thousands_separator
    decimal_point = _config.DECIMAL_POINT if decimal_point is ... else decimal_point

    if isinstance(number, int):
        if thousands_separator:
            string = format(number, ",").replace(",", thousands_separator)
        else:
            string = str(number)
        return string

    if isinstance(number, float):
        if not decimal_point:
            raise ValueError("Converting float to string requires decimal point to be known")

        if thousands_separator:
            string = format(number, ",")
        else:
            string = str(number)

        # swap , and . simultaneously for thousands_separator and decimal_point
        string = string.translate(str.maketrans({",": thousands_separator, ".": decimal_point}))

        return string



import locale

DECIMAL_POINT = locale.localeconv()["decimal_point"]
THOUSANDS_SEPARATOR = locale.localeconv()["thousands_sep"]


def set_decimal_point_and_thousands_separator() -> None:
    """Set `DECIMAL_POINT` and `THOUSHANDS_SEPARATOR` based on the system locale.

    These values are used to parse analysis result text files.

    Returns
    -------
    None

    Warnings
    --------
    This function is not thread-safe due to the use of `locale.setlocale`.
    Normal use of ZOSPy is unlikely to cause problems, but creating new threads prior to
    importing ZOSPy may cause issues. See also https://docs.python.org/3/library/locale.html#locale.setlocale
    for more information.
    """
    global DECIMAL_POINT
    global THOUSANDS_SEPARATOR

    loc = locale.getlocale()  # get and save current locale
    locale.setlocale(locale.LC_ALL, "")
    DECIMAL_POINT = locale.localeconv()["decimal_point"]
    THOUSANDS_SEPARATOR = locale.localeconv()["thousands_sep"]
    locale.setlocale(locale.LC_ALL, loc)  # restore saved locale



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from zospy.api._ZOSAPI.Analysis.Settings import (
    IAS_,
    AxisType,
    IAS_Surface,
    IAS_Wavelength,
)

__all__ = ("IAS_GrinProfile", "IAS_InternalTransmissionvsWavelength")

class IAS_GrinProfile(IAS_):
    @property
    def MaximumIndex(self) -> float: ...
    @property
    def MaximumX(self) -> float: ...
    @property
    def MaximumY(self) -> float: ...
    @property
    def MaximumZ(self) -> float: ...
    @property
    def MinimumIndex(self) -> float: ...
    @property
    def MinimumX(self) -> float: ...
    @property
    def MinimumY(self) -> float: ...
    @property
    def MinimumZ(self) -> float: ...
    @property
    def ShowIndexVs(self) -> AxisType: ...
    @property
    def Surface(self) -> IAS_Surface: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @property
    def X_Value(self) -> float: ...
    @property
    def Y_Value(self) -> float: ...
    @property
    def Z_Value(self) -> float: ...
    @MaximumIndex.setter
    def MaximumIndex(self, value: float) -> None: ...
    @MaximumX.setter
    def MaximumX(self, value: float) -> None: ...
    @MaximumY.setter
    def MaximumY(self, value: float) -> None: ...
    @MaximumZ.setter
    def MaximumZ(self, value: float) -> None: ...
    @MinimumIndex.setter
    def MinimumIndex(self, value: float) -> None: ...
    @MinimumX.setter
    def MinimumX(self, value: float) -> None: ...
    @MinimumY.setter
    def MinimumY(self, value: float) -> None: ...
    @MinimumZ.setter
    def MinimumZ(self, value: float) -> None: ...
    @ShowIndexVs.setter
    def ShowIndexVs(self, value: AxisType) -> None: ...
    @X_Value.setter
    def X_Value(self, value: float) -> None: ...
    @Y_Value.setter
    def Y_Value(self, value: float) -> None: ...
    @Z_Value.setter
    def Z_Value(self, value: float) -> None: ...

class IAS_InternalTransmissionvsWavelength(IAS_):
    @property
    def Glass1(self) -> str: ...
    @property
    def Glass2(self) -> str: ...
    @property
    def Glass3(self) -> str: ...
    @property
    def Glass4(self) -> str: ...
    @property
    def MaximumTransmission(self) -> float: ...
    @property
    def MaximumWave(self) -> float: ...
    @property
    def MinimumTransmission(self) -> float: ...
    @property
    def MinimumWave(self) -> float: ...
    @property
    def Thickness(self) -> float: ...
    @Glass1.setter
    def Glass1(self, value: str) -> None: ...
    @Glass2.setter
    def Glass2(self, value: str) -> None: ...
    @Glass3.setter
    def Glass3(self, value: str) -> None: ...
    @Glass4.setter
    def Glass4(self, value: str) -> None: ...
    @MaximumTransmission.setter
    def MaximumTransmission(self, value: float) -> None: ...
    @MaximumWave.setter
    def MaximumWave(self, value: float) -> None: ...
    @MinimumTransmission.setter
    def MinimumTransmission(self, value: float) -> None: ...
    @MinimumWave.setter
    def MinimumWave(self, value: float) -> None: ...
    @Thickness.setter
    def Thickness(self, value: float) -> None: ...



"""Zemax OpticStudio analyses from the Reports category."""

from __future__ import annotations

import os
import re
from tempfile import mkstemp
from typing import Any

import numpy as np
import pandas as pd

from zospy.analyses.base import AnalysisResult, AttrDict, OnComplete, new_analysis
from zospy.api import constants
from zospy.zpcore import OpticStudioSystem

# TODO use value from zospy.api._config
REFLOAT = re.compile(r"^[-+]?(?:(?:\d*\.\d+)|(?:\d+\.?))(?:[Ee][+-]?\d+)?$")
RECOMMASEP = re.compile(r"(?<=\d),(?=\d)")


def _structure_surface_data_result(line_list: list[str]) -> pd.Series:
    """Structures the result of a surface data analysis.

    Parameters
    ----------
    line_list: list of str
        The line list obtained by reading in the results

    Returns
    -------
    pd.Series
        The results structured in a Series
    """
    res = pd.Series(index=pd.MultiIndex([[]] * 2, [[]] * 2), dtype=object)

    lm = pd.Series(
        index=[
            "Thickness",
            "Index of Refraction",
            "Best Fit Glass",
            "Surface Powers (as situated)",
            "Surface Powers (in air)",
            "Shape Factor",
        ],
        data=[0] * 6,
        dtype=int,
    )

    for item in lm.index:
        for num, line in enumerate(line_list):
            if line.lstrip().startswith(item):
                lm.loc[item] = num
                break

    general_lm = ["File", "Title", "Date", "Comment", "Lens units"]
    surface_lm = ["Thickness", "Diameter", "Y Edge Thick", "X Edge Thick"]
    ri_lm = ["nd", "Abbe", "dPgF", "Best Fit Glass"]

    for line in line_list[: lm["Thickness"]]:
        for item in general_lm:
            if line.lstrip().startswith(item):
                val = line.split(":")[-1].strip()

                fval = val.replace(",", ".")  # to enable conversion to float
                if REFLOAT.match(fval):
                    val = float(fval)

                res.loc["General", item] = val

    for line in line_list[lm["Thickness"] : lm["Index of Refraction"]]:
        for item in surface_lm:
            if line.lstrip().startswith(item):
                val = line.split(":")[-1].strip()

                fval = val.replace(",", ".")  # to enable conversion to float
                if REFLOAT.match(fval):
                    val = float(fval)

                res.loc["Surface", item] = val

    for line in line_list[lm["Index of Refraction"] : lm["Best Fit Glass"] + 1]:  # + 1 as bfg should be included
        for item in ri_lm:
            if line.lstrip().startswith(item):
                val = line.split(":")[-1].strip()

                fval = val.replace(",", ".")  # to enable conversion to float
                if REFLOAT.match(fval):
                    val = float(fval)

                res.loc["IndexOfRefraction", item] = val

    for line in line_list[
        lm["Best Fit Glass"] + 1 : lm["Surface Powers (as situated)"]
    ]:  # + 1 as bfg should be excluded
        if len(line.split("\t")) == 3:
            _, wl, ri = line.split("\t")
            fwl = wl.strip().replace(",", ".")
            if REFLOAT.match(fwl):
                wl = float(fwl)
            else:
                continue  # is not an actual wavelength

            ri = ri.strip().replace(",", ".")
            if REFLOAT.match(ri):
                ri = float(ri)

            res.loc["IndexOfRefractionPerWavelength", wl] = ri

    for line in line_list[lm["Surface Powers (as situated)"] + 1 : lm["Surface Powers (in air)"]]:
        if len(line.split(":")) == 2:
            param, val = line.split(":")
            param = re.sub(r" +", " ", param).strip()

            fval = val.strip().replace(",", ".")
            if REFLOAT.match(fval):
                val = float(fval)

            res.loc["SurfacePowerAsSituated", param] = val

    for line in line_list[lm["Surface Powers (in air)"] + 1 : lm["Shape Factor"]]:
        if len(line.split(":")) == 2:
            param, val = line.split(":")
            param = re.sub(r" +", " ", param).strip()

            fval = val.strip().replace(",", ".")
            if REFLOAT.match(fval):
                val = float(fval)

            res.loc["SurfacePowerInAir", param] = val

    for line in line_list[lm["Shape Factor"] :]:
        if len(line.split(":")) == 2:
            param, val = line.split(":")
            param = re.sub(r" +", " ", param).strip()

            val = val.strip()
            fval = val.replace(",", ".")
            if REFLOAT.match(fval):
                val = float(fval)

            res.loc["Other", param] = val

    return res


def surface_data(
    oss: OpticStudioSystem,
    surface: int,
    oncomplete: OnComplete | str = OnComplete.Close,
    cfgoutfile: str | None = None,
    txtoutfile: str | None = None,
) -> AnalysisResult:
    """Wrapper around the OpticStudio Surface Data Analysis.

    Due to limitations in the ZOS-API, the output is obtained by writing the OpticStudio results to a file and
    subsequently reading in this file

    Parameters
    ----------
    oss: zospy.core.OpticStudioSystem
        A ZOSPy OpticStudioSystem instance. Should be sequential.
    surface: int
        The surface number that is to be analyzed
    oncomplete: OnComplete | str
        Defines behaviour upon completion of the analysis. Should be one of ['Close', 'Release', 'Sustain']. If 'Close',
        the analysis will be closed after completion. If 'Release', the analysis will remain open in OpticStudio, but
        the link with python will be destroyed. If 'Sustain' the analysis will be kept open in OpticStudio and the link
        with python will be sustained. To enable interaction when oncomplete == 'Sustain', the OpticStudio Analysis
        instance will be available in the returned AnalysisResult through AnalysisResult.Analysis. Defaults to 'Close'.
    cfgoutfile: str | None
        The configuration file to which the current configuration is saved. If None, a temporary file will be created
        and deleted. If string, it should be a full system path with '.CFG' as extension, after which the file will be
        saved in that location. (Allowing usage of this configuration file using surface_data_fromcfg()). Defaults to
        None.
    txtoutfile: str | None
        The textfile to which the OpticStudio output is saved. If None, a temporary file will be created and deleted. If
        string, it should be a full system path with '.txt' as extension, after which the file will be saved in that
        location. Defaults to None.

    Returns
    -------
    AnalysisResult
        A SurfaceDataSettings analysis result. Next to the standard data, the raw text return obtained from the analysis
        will be present under 'RawTextData', the cfgoutfile under 'CfgOutFile', and the txtoutfile under 'TxtOutFile'.
    """
    analysis_type = constants.Analysis.AnalysisIDM.SurfaceDataSettings

    if cfgoutfile is None:
        fd, cfgoutfile = mkstemp(suffix=".CFG", prefix="zospy_")
        os.close(fd)
        cleancfg = True
    else:
        if not cfgoutfile.endswith(".CFG"):
            raise ValueError('cfgfile should end with ".CFG"')
        cleancfg = False

    if txtoutfile is None:
        fd, txtoutfile = mkstemp(suffix=".txt", prefix="zospy_")
        os.close(fd)
        cleantxt = True
    else:
        if not txtoutfile.endswith(".txt"):
            raise ValueError('txtfile should end with ".txt"')
        cleantxt = False

    analysis = new_analysis(oss, analysis_type)

    # Modify surface in the settings file
    an_sett = analysis.GetSettings()
    an_sett.SaveTo(cfgoutfile)

    settingsbstr = b"".join(open(cfgoutfile, "rb").readlines())
    settingsbarr = bytearray(settingsbstr)
    settingsbarr[20] = surface  # 20 maps to the selected surface

    with open(cfgoutfile, "wb") as bfile:
        bfile.write(settingsbarr)

    an_sett.LoadFrom(cfgoutfile)

    # Run analysis
    analysis.ApplyAndWaitForCompletion()

    # Get results
    analysis.Results.GetTextFile(txtoutfile)
    line_list = [line for line in open(txtoutfile, "r", encoding=oss._ZOS.get_txtfile_encoding())]
    data = _structure_surface_data_result(line_list)

    # Get headerdata, metadata and messages
    headerdata = analysis.get_header_data()
    metadata = analysis.get_metadata()
    messages = analysis.get_messages()

    # Manually create settings as they cannot be accessed
    settings = pd.Series(name="Settings", dtype=object)
    settings.loc["Surface"] = surface

    # Create output
    result = AnalysisResult(
        analysistype=str(analysis_type),
        data=data,
        settings=settings,
        metadata=metadata,
        headerdata=headerdata,
        messages=messages,
        RawTextData=line_list,
        CgfOutFile=cfgoutfile,
        TxtOutFile=txtoutfile,
    )  # Set additional params

    # cleanup if needed
    if cleancfg:
        os.remove(cfgoutfile)
    if cleantxt:
        os.remove(txtoutfile)

    return analysis.complete(oncomplete, result)


def surface_data_fromcfg(
    oss: OpticStudioSystem, cfgfile: str, oncomplete: OnComplete | str = OnComplete.Close, txtoutfile: str | None = None
) -> AnalysisResult:
    """Wrapper around the OpticStudio Surface Data Analysis that uses a predefined configuration file.

    Due to limitations in the ZOS-API, the output is obtained by writing the OpticStudio results to a file and
    subsequently reading in this file

    Parameters
    ----------
    oss: zospy.core.OpticStudioSystem
        A ZOSPy OpticStudioSystem instance. Should be sequential.
    cfgfile: str
        The configuration file that is to be used. Should be a full system path with '.CFG' as extension.
    oncomplete: OnComplete | str
        Defines behaviour upon completion of the analysis. Should be one of ['Close', 'Release', 'Sustain']. If 'Close',
        the analysis will be closed after completion. If 'Release', the analysis will remain open in OpticStudio, but
        the link with python will be destroyed. If 'Sustain' the analysis will be kept open in OpticStudio and the link
        with python will be sustained. To enable interaction when oncomplete == 'Sustain', the OpticStudio Analysis
        instance will be available in the returned AnalysisResult through AnalysisResult.Analysis. Defaults to 'Close'.
    txtoutfile: str | None
        The textfile to which the Zemax output is saved. If None, a temporary file will be created and deleted. If
        string, it should be a full system path with '.txt' as extension, after which the file will be saved in that
        location. Defaults to None.

    Returns
    -------
    AnalysisResult
        A SurfaceDataSettings analysis result. Next to the standard data, the used cfgfile will be present under
        'UsedCfgFile', raw text return under 'RawTextData' and the txtoutfile under 'TxtOutFile'.
    """
    analysis_type = constants.Analysis.AnalysisIDM.SurfaceDataSettings

    if not cfgfile.endswith(".CFG"):
        raise ValueError('cfgfile should end with ".CFG"')

    if txtoutfile is None:
        fd, txtoutfile = mkstemp(suffix=".txt", prefix="zospy_")
        os.close(fd)
        cleantxt = True
    else:
        if not txtoutfile.endswith(".txt"):
            raise ValueError('txtfile should end with ".txt"')
        cleantxt = False

    analysis = new_analysis(oss, analysis_type)

    # Load settings
    analysis.Settings.LoadFrom(cfgfile)

    # Run analysis
    analysis.ApplyAndWaitForCompletion()

    # Get results
    analysis.Results.GetTextFile(txtoutfile)
    line_list = [line for line in open(txtoutfile, "r", encoding=oss._ZOS.get_txtfile_encoding())]
    data = _structure_surface_data_result(line_list)

    # Get headerdata, metadata and messages
    headerdata = analysis.get_header_data()
    metadata = analysis.get_metadata()
    messages = analysis.get_messages()

    # Manually create settings as they cannot be accessed
    # TODO replace by None
    settings = pd.Series(name="Settings", dtype=object)

    # Create output
    result = AnalysisResult(
        analysistype=str(analysis_type),
        data=data,
        settings=settings,
        metadata=metadata,
        headerdata=headerdata,
        messages=messages,
        UsedCfgFile=cfgfile,
        RawTextData=line_list,
        TxtOutFile=txtoutfile,
    )  # Set additional params

    # cleanup
    if cleantxt:
        os.remove(txtoutfile)

    return analysis.complete(oncomplete, result)


def _structure_system_data_result(line_list: list[str]) -> dict[str, Any]:
    """Structures the result of a system data report.

    Parameters
    ----------
    line_list: list[str]
        The line list obtained by reading in the results

    Returns
    -------
    dict
        The results structured in a dictionary
    """
    # Create output dict
    res = AttrDict()

    # Add header
    hdr = line_list[0].strip().replace("\ufeff", "")
    res["Header"] = hdr

    # Register landmarks
    landmarks = ["GENERAL LENS DATA", "Fields", "Vignetting Factors", "Wavelengths", "Predicted coordinate ABCD matrix"]
    comp = re.compile(rf'^({"|".join(landmarks)})', re.IGNORECASE)
    inds = {
        comp.match(line).group().lower(): {"rnum": rnum, "name": comp.match(line).group()}
        for rnum, line in enumerate(line_list)
        if comp.match(line)
    }

    order_of_appearance = [item[0] for item in sorted(inds.items(), key=lambda x: x[1]["rnum"])]

    # Add general lens data
    target = "general lens data"
    gld = pd.Series(dtype=object)
    if target in inds:
        section_start = inds[target]["rnum"] + 1  # + 1 to ignore section title

        # Get endpoint
        if order_of_appearance.index(target) == len(inds) - 1:
            section_end = len(line_list)
        else:
            target2 = order_of_appearance[order_of_appearance.index(target) + 1]
            section_end = inds[target2]["rnum"]

        for line in line_list[section_start:section_end]:
            if line.strip() == "":
                continue

            # Get param and value, strip whitespaces
            param, val = line.split(":")
            param = re.sub(r"\s+", " ", param).strip()

            # convert value to float if needed
            val = re.sub(r"\s+", " ", val).strip()
            val = RECOMMASEP.sub(".", val)
            if REFLOAT.match(val):
                if "." in val or "e" in val:
                    val = float(val)
                else:
                    val = int(val)

            # add
            ii = 0
            while param in gld.index:
                ii += 1
                param = f"{param} {ii}"
            gld.loc[param] = val
    res["GeneralLensData"] = gld

    # Fields, Wavelengths
    for target in ["fields", "wavelengths"]:
        info = pd.Series(dtype=object)
        data = pd.DataFrame()
        if target in inds:
            section_start = inds[target]["rnum"]

            # Get endpoint
            if order_of_appearance.index(target) == len(inds) - 1:
                section_end = len(line_list)
            else:
                target2 = order_of_appearance[order_of_appearance.index(target) + 1]
                section_end = inds[target2]["rnum"]

            for ii, line in enumerate(line_list[section_start:section_end]):
                if line.strip().startswith("#"):
                    ht_loc = section_start + ii
                    break
            else:
                ht_loc = None

            param_end = ht_loc if ht_loc is not None else section_end
            for line in line_list[section_start:param_end]:
                if line.strip() == "":
                    continue

                # Get param and value, strip whitespaces
                param, val = line.split(":")
                param = re.sub(r"\s+", " ", param).strip()

                # convert value to float if needed
                val = re.sub(r"\s+", " ", val).strip()
                val = RECOMMASEP.sub(".", val)
                if REFLOAT.match(val):
                    if "." in val:
                        val = float(val)
                    else:
                        val = int(val)

                # add
                ii = 0
                while param in gld.index:
                    ii += 1
                    param = f"{param} {ii}"
                info.loc[param] = val

            if ht_loc is not None:
                columns = re.sub(r"\s+", " ", line_list[ht_loc]).strip().split(" ")
                data = []
                for ii in range(ht_loc + 1, section_end):
                    if line_list[ii].strip() == "":
                        continue
                    items = re.sub(r"\s+", " ", line_list[ii]).strip().split(" ")
                    values = []
                    for item in items:
                        val = RECOMMASEP.sub(".", item)
                        if REFLOAT.match(val):
                            if "." in val:
                                val = float(val)
                            else:
                                val = int(val)
                        values.append(val)
                    data.append(values)
                data = pd.DataFrame(columns=columns, data=data)
            res[inds[target]["name"]] = AttrDict(Info=info, Data=data)

    # Vignetting factors
    target = "vignetting factors"
    if target in inds:
        section_start = inds[target]["rnum"] + 1  # + 1 to ignore section title

        # Get endpoint
        if order_of_appearance.index(target) == len(inds) - 1:
            section_end = len(line_list)
        else:
            target2 = order_of_appearance[order_of_appearance.index(target) + 1]
            section_end = inds[target2]["rnum"]

        for ii, line in enumerate(line_list[section_start:section_end]):
            if line.strip().startswith("#"):
                ht_loc = section_start + ii
                break
        else:
            ht_loc = None

        if ht_loc is not None:
            columns = re.sub(r"\s+", " ", line_list[ht_loc]).strip().split(" ")
            data = []
            for ii in range(ht_loc + 1, section_end):
                if line_list[ii].strip() == "":
                    continue
                items = re.sub(r"\s+", " ", line_list[ii]).strip().split(" ")
                values = []
                for item in items:
                    val = RECOMMASEP.sub(".", item)
                    if REFLOAT.match(val):
                        if "." in val:
                            val = float(val)
                        else:
                            val = int(val)
                    values.append(val)
                data.append(values)
            data = pd.DataFrame(columns=columns, data=data)
        res["VignettingFactors"] = data

    # Predicted ABCD matrix
    target = "predicted coordinate abcd matrix"
    abcd = pd.Series(dtype=object)
    if target in inds:
        section_start = inds[target]["rnum"] + 1  # + 1 to ignore section title

        # Get endpoint
        if order_of_appearance.index(target) == len(inds) - 1:
            section_end = len(line_list)
        else:
            target2 = order_of_appearance[order_of_appearance.index(target) + 1]
            section_end = inds[target2]["rnum"]

        for line in line_list[section_start:section_end]:
            if line.strip() == "":
                continue

            # Get param and value, strip whitespaces
            param, val = line.split("=")
            param = re.sub(r"\s+", " ", param).strip()

            # convert value to float if needed
            val = re.sub(r"\s+", " ", val).strip()
            val = RECOMMASEP.sub(".", val)
            if REFLOAT.match(val):
                if "." in val:
                    val = float(val)
                else:
                    val = int(val)

            # add
            ii = 0
            while param in gld.index:
                ii += 1
                param = f"{param} {ii}"
            abcd.loc[param] = val
        res["PredictedCoordinateABCDMatrix"] = abcd

        return res


def system_data(
    oss: OpticStudioSystem, oncomplete: OnComplete | str = OnComplete.Close, txtoutfile: str | None = None
) -> AnalysisResult:
    """Wrapper around the OpticStudio System Data Analysis.

    Due to limitations in the ZOS-API, the output is obtained by writing the OpticStudio results to a file and
    subsequently reading in this file

    Parameters
    ----------
    oss: zospy.core.OpticStudioSystem
        A ZOSPy OpticStudioSystem instance. Should be sequential.
    oncomplete: OnComplete | str
        Defines behaviour upon completion of the analysis. Should be one of ['Close', 'Release', 'Sustain']. If 'Close',
        the analysis will be closed after completion. If 'Release', the analysis will remain open in OpticStudio, but
        the link with python will be destroyed. If 'Sustain' the analysis will be kept open in OpticStudio and the link
        with python will be sustained. To enable interaction when oncomplete == 'Sustain', the OpticStudio Analysis
        instance will be available in the returned AnalysisResult through AnalysisResult.Analysis. Defaults to 'Close'.
    txtoutfile: str or None
        The textfile to which the OpticStudio output is saved. If None, a temporary file will be created and deleted. If
        string, it should be a full system path with '.txt' as extension, after which the file will be saved in that
        location. Defaults to None.

    Returns
    -------
    AnalysisResult
        A SystemData analysis result. Next to the standard data, the raw text return obtained from the analysis
        will be present under 'RawTextData', the cfgoutfile under 'CfgOutFile', and the txtoutfile under 'TxtOutFile'.
    """
    analysis_type = constants.Analysis.AnalysisIDM.SystemData

    if txtoutfile is None:
        fd, txtoutfile = mkstemp(suffix=".txt", prefix="zospy_")
        os.close(fd)
        cleantxt = True
    else:
        if not txtoutfile.endswith(".txt"):
            raise ValueError('txtfile should end with ".txt"')
        cleantxt = False

    analysis = new_analysis(oss, analysis_type)

    # Run analysis
    analysis.ApplyAndWaitForCompletion()

    # Get results
    analysis.Results.GetTextFile(txtoutfile)
    line_list = [line for line in open(txtoutfile, "r", encoding=oss._ZOS.get_txtfile_encoding())]
    data = _structure_system_data_result(line_list)

    # Get headerdata, metadata and messages
    headerdata = analysis.get_header_data()
    metadata = analysis.get_metadata()
    messages = analysis.get_messages()

    # Create output
    result = AnalysisResult(
        analysistype=str(analysis_type),
        data=data,
        settings=None,
        metadata=metadata,
        headerdata=headerdata,
        messages=messages,
        RawTextData=line_list,
        TxtOutFile=txtoutfile,
    )  # Set additional params

    # cleanup if needed
    if cleantxt:
        os.remove(txtoutfile)

    return analysis.complete(oncomplete, result)


def _structure_cardinal_point_result(line_list: list[str]) -> pd.Series:
    """Structures the result of a cardinal point analysis.

    Parameters
    ----------
    line_list: list[str]
        The line list obtained by reading in the results

    Returns
    -------
    pd.Series
        The results structured in a Series
    """
    res = pd.Series(index=pd.MultiIndex([[]] * 2, [[]] * 2), dtype=object)

    # Determine landmarks
    idx = [
        "Starting surface",
        "Ending surface",
        "Object space positions",
        "Focal Length",
        "Anti-Nodal Planes",
        "Error computing cardinal points",
    ]
    lm = pd.Series(index=idx, data=[pd.NA] * len(idx), dtype=object)

    for item in lm.index:
        for num, line in enumerate(line_list):
            if line.lstrip().startswith(item):
                lm.loc[item] = num
                break

    general_lm = ["File", "Title", "Date", "Wavelength", "Orientation", "Lens units"]
    surface_lm = ["Starting surface", "Ending surface"]
    carpoints_lm = [
        "Focal Length",
        "Focal Planes",
        "Principal Planes",
        "Anti-Principal Planes",
        "Nodal Planes",
        "Anti-Nodal Planes",
    ]

    # Get general info
    for line in line_list[: lm["Object space positions"]]:
        for item in general_lm:
            if line.lstrip().startswith(item):
                val = line.split(":")[-1].strip()

                fval = val.replace(",", ".")  # to enable conversion to float
                if REFLOAT.match(fval):
                    val = float(fval)

                res.loc["General", item] = val

    space1 = space2 = "Space"
    for option in ["Focal Length", "Error computing cardinal points"]:
        try:
            res.loc["General", "Info"] = "\n".join(
                [
                    item.strip()
                    for item in line_list[lm["Object space positions"] : lm[option] - 1]
                    if item.strip() != ""
                ]
            )
            _, space1, space2 = line_list[lm[option] - 1].split("\t")
            space1 = space1.strip()
            space2 = space2.strip()

            break
        except TypeError:
            continue
    else:
        res.loc["General", "Info"] = "ZOSPy: Cannot determine spaces"

    for line in line_list[lm["Starting surface"] : lm["Ending surface"] + 1]:
        for item in surface_lm:
            if line.lstrip().startswith(item):
                val = line.split(":")[-1].strip()

                fval = val.replace(",", ".")  # to enable conversion to float
                if REFLOAT.match(fval):
                    val = float(fval)

                res.loc["Surface", item] = val

    try:
        for line in line_list[lm["Focal Length"] : lm["Anti-Nodal Planes"] + 1]:
            for item in carpoints_lm:
                if line.lstrip().startswith(item):
                    val1, val2 = line.split(":")[-1].strip().split("\t")
                    val1 = val1.strip()
                    val2 = val2.strip()

                    fval1 = val1.replace(",", ".")  # to enable conversion to float
                    if REFLOAT.match(fval1):
                        val1 = float(fval1)

                    fval2 = val2.replace(",", ".")  # to enable conversion to float
                    if REFLOAT.match(fval2):
                        val2 = float(fval2)

                    res.loc[space1, item] = val1
                    res.loc[space2, item] = val2
    except TypeError:
        for item in carpoints_lm:
            res.loc[space1, item] = np.nan
            res.loc[space2, item] = np.nan
    res = res.loc[res.index.get_level_values(0).unique()]

    return res


def cardinal_points(
    oss: OpticStudioSystem,
    surface_1: int,
    surface_2: int,
    oncomplete: OnComplete | str = OnComplete.Close,
    cfgoutfile: str | None = None,
    txtoutfile: str | None = None,
) -> AnalysisResult:
    """Wrapper around the OpticStudio Cardinal Point Analysis.

    Parameters
    ----------
    oss: zospy.core.OpticStudioSystem
        A ZOSPy OpticStudioSystem instance. Should be sequential.
    surface_1: int
        The surface number corresponding to the first surface of the analyzed system
    surface_2: int
        The surface number corresponding to the last surface of the analyzed system
    oncomplete: OnComplete | str
        Defines behaviour upon completion of the analysis. Should be one of ['Close', 'Release', 'Sustain']. If 'Close',
        the analysis will be closed after completion. If 'Release', the analysis will remain open in OpticStudio, but
        the link with python will be destroyed. If 'Sustain' the analysis will be kept open in OpticStudio and the link
        with python will be sustained. To enable interaction when oncomplete == 'Sustain', the OpticStudio Analysis
        instance will be available in the returned AnalysisResult through AnalysisResult.Analysis. Defaults to 'Close'.
    cfgoutfile: str | None
        The configuration file to which the current configuration is saved. If None, a temporary file will be created
        and deleted. If string, it should be a full system path with '.CFG' as extension, after which the file will be
        saved in that location. (Allowing usage of this configuration file using surface_data_fromcfg()). Defaults to
        None.
    txtoutfile: str | None
        The textfile to which the OpticStudio output is saved. If None, a temporary file will be created and deleted. If
        string, it should be a full system path with '.txt' as extension, after which the file will be saved in that
        location. Defaults to None.

    Returns
    -------
    AnalysisResult
        A SurfaceDataSettings analysis result. Next to the standard data, the raw text return obtained from the analysis
        will be present under 'RawTextData', the cfgoutfile under 'CfgOutFile', and the txtoutfile under 'TxtOutFile'.
    """
    analysis_type = constants.Analysis.AnalysisIDM.CardinalPoints

    if surface_1 > surface_2:
        raise ValueError("Surface 1 cannot be higher than Surface 2")

    if cfgoutfile is None:
        fd, cfgoutfile = mkstemp(suffix=".CFG", prefix="zospy_")
        os.close(fd)
        cleancfg = True
    else:
        if not cfgoutfile.endswith(".CFG"):
            raise ValueError('cfgoutfile should end with ".CFG"')
        cleancfg = False

    if txtoutfile is None:
        fd, txtoutfile = mkstemp(suffix=".txt", prefix="zospy_")
        os.close(fd)
        cleantxt = True
    else:
        if not txtoutfile.endswith(".txt"):
            raise ValueError('txtoutfile should end with ".txt"')
        cleantxt = False

    analysis = new_analysis(oss, analysis_type)

    # Modify surface in the settings file
    analysis_settings = analysis.Settings
    analysis_settings.SaveTo(cfgoutfile)

    settingsbstr = b"".join(open(cfgoutfile, "rb").readlines())
    settingsbarr = bytearray(settingsbstr)
    settingsbarr[20] = surface_1  # byte 20 maps to the first surface
    settingsbarr[24] = surface_2  # byte 24 maps to the first surface

    with open(cfgoutfile, "wb") as bfile:
        bfile.write(settingsbarr)

    analysis_settings.LoadFrom(cfgoutfile)

    # Run analysis
    analysis.ApplyAndWaitForCompletion()

    # Get results
    analysis.Results.GetTextFile(txtoutfile)
    line_list = [line for line in open(txtoutfile, "r", encoding=oss._ZOS.get_txtfile_encoding())]
    data = _structure_cardinal_point_result(line_list)

    # Get headerdata, metadata and messages
    headerdata = analysis.get_header_data()
    metadata = analysis.get_metadata()
    messages = analysis.get_messages()

    # Manually create settings as they cannot be accessed
    settings = pd.Series(name="Settings", dtype=object)
    settings.loc["Surface1"] = surface_1
    settings.loc["Surface2"] = surface_2

    # Create output
    result = AnalysisResult(
        analysistype=str(analysis_type),
        data=data,
        settings=settings,
        metadata=metadata,
        headerdata=headerdata,
        messages=messages,
        RawTextData=line_list,
        CgfOutFile=cfgoutfile,
        TxtOutFile=txtoutfile,
    )  # Set additional params

    # cleanup if needed
    if cleancfg:
        os.remove(cfgoutfile)
    if cleantxt:
        os.remove(txtoutfile)

    return analysis.complete(oncomplete, result)


def cardinal_points_fromcfg(
    oss: OpticStudioSystem, cfgfile: str, oncomplete: OnComplete | str = OnComplete.Close, txtoutfile: str | None = None
) -> AnalysisResult:
    """Wrapper around the OpticStudio Cardinal Point Analysis that uses a predefined configuration file.

    Parameters
    ----------
    oss: zospy.core.OpticStudioSystem
        A ZOSPy OpticStudioSystem instance. Should be sequential.
    cfgfile: str
        The configuration file that is to be used. Should be a full system path with '.CFG' as extension.
    oncomplete: OnComplete | str
        Defines behaviour upon completion of the analysis. Should be one of ['Close', 'Release', 'Sustain']. If 'Close',
        the analysis will be closed after completion. If 'Release', the analysis will remain open in OpticStudio, but
        the link with python will be destroyed. If 'Sustain' the analysis will be kept open in OpticStudio and the link
        with python will be sustained. To enable interaction when oncomplete == 'Sustain', the OpticStudio Analysis
        instance will be available in the returned AnalysisResult through AnalysisResult.Analysis. Defaults to 'Close'.
    txtoutfile: str | None
        The textfile to which the OpticStudio output is saved. If None, a temporary file will be created and deleted. If
        string, it should be a full system path with '.txt' as extension, after which the file will be saved in that
        location. Defaults to None.

    Returns
    -------
    AnalysisResult
        A SurfaceDataSettings analysis result. Next to the standard data, the used cfgfile will be present under
        'UsedCfgFile', raw text return under 'RawTextData' and the txtoutfile under 'TxtOutFile'.
    """
    analysis_type = constants.Analysis.AnalysisIDM.CardinalPoints

    if not cfgfile.endswith(".CFG"):
        raise ValueError('cfgfile should end with ".CFG"')

    if txtoutfile is None:
        fd, txtoutfile = mkstemp(suffix=".txt", prefix="zospy_")
        os.close(fd)
        cleantxt = True
    else:
        if not txtoutfile.endswith(".txt"):
            raise ValueError('txtfile should end with ".txt"')
        cleantxt = False

    analysis = new_analysis(oss, analysis_type)

    # Load the settings file
    analysis.Settings.LoadFrom(cfgfile)

    # Run analysis
    analysis.ApplyAndWaitForCompletion()

    # Get results
    analysis.Results.GetTextFile(txtoutfile)
    line_list = [line for line in open(txtoutfile, "r", encoding=oss._ZOS.get_txtfile_encoding())]
    data = _structure_cardinal_point_result(line_list)

    # Get headerdata, metadata and messages
    headerdata = analysis.get_header_data()
    metadata = analysis.get_metadata()
    messages = analysis.get_messages()

    # Manually create settings as they cannot be accessed
    # TODO replace by None
    settings = pd.Series(name="Settings", dtype=object)

    # Create output
    result = AnalysisResult(
        analysistype=str(analysis_type),
        data=data,
        settings=settings,
        metadata=metadata,
        headerdata=headerdata,
        messages=messages,
        UsedCfgFile=cfgfile,
        RawTextData=line_list,
        TxtOutFile=txtoutfile,
    )  # Set additional params

    # cleanup if needed
    if cleantxt:
        os.remove(txtoutfile)

    return analysis.complete(oncomplete, result)



import re
from collections.abc import MutableMapping

import numpy as np
import pandas as pd


def flatten_dict(unflattend_dict, parent_key="", sep=".", keep_unflattend=False):
    """Flattens an nested dictionary to a one-level dictionary.

    Parameters
    ----------
    unflattend_dict: dict or MutableMapping
        A dictionary with nested dictionaries
    parent_key: str
        A key to which the flattened keys are added. Mainly present to support flatting of subdicts, should be '' in
        most primary calls.. Defaults to ''.
    sep: str
        The separator used for the flat items. Defaults to '.'
    keep_unflattend: bool
        Whether the keys, value pairs of nested dicts or MutableMappings should remain present in the output or not.
        Defaults to False

    Returns
    -------
    dict
        A dictionary with one (flattened) key for a value.
    """
    items = []
    for key, value in unflattend_dict.items():
        new_key = parent_key + sep + key if parent_key else key
        if isinstance(value, MutableMapping):
            if keep_unflattend:
                items.append((new_key, value))
            items.extend(flatten_dict(value, new_key, sep=sep).items())
        else:
            items.append((new_key, value))
    return dict(items)


def flatten_dlltreedict(dlltreedict):
    """Flattens the set of nested dictionaries that represent a dll tree.

    For instance, it can flatten the dictionary tree obtained from ZOSAPINestedNameSpaces.json.

    Parameters
    ----------
    dlltreedict: dict
        A dictionary with nested dictionaries

    Returns
    -------
    list:
        A list with the items of the nested dictionaries.
    """
    flatdict = flatten_dict(dlltreedict, parent_key="", sep=".", keep_unflattend=True)
    flatlist = list(flatdict.keys())
    return flatlist


def unpack_dataseries(dataseries):
    """Unpacks a dataseries in a dataframe.

    Parameters
    ----------
    dataseries: Zemax OpticStudio DataSeries
        A Zemax Opticstudio Dataseries.

    Returns
    -------
    pd.DataFrame
        A DataFrame holding all data and labels
    """
    columns = pd.MultiIndex.from_product(
        [[dataseries.Description], list(dataseries.SeriesLabels)], names=["Description", "SeriesLabels"]
    )
    index = np.array(list(dataseries.XData.Data))
    data = np.array(list(dataseries.YData.Data)).reshape(dataseries.YData.Rows, dataseries.YData.Cols)
    df = pd.DataFrame(columns=columns, index=index, data=data)  # ToDo evaluate
    df.index.name = dataseries.XLabel
    return df


def unpack_datagrid(datagrid):
    """Unpacks an OpticStudio datagrid.

    Parameters
    ----------
    datagrid: OpticStudio DataGrid
        An OpticStudio DataGrid

    Returns
    -------
    pd.DataFrame
        A DataFrame containing the grid and the spacing on the rows and columns. The X- and YLabels are assigned to the
        df.columns.name and df.index.name
    """
    values = np.array(list(datagrid.Values))
    values = values.reshape(datagrid.Ny, datagrid.Nx)

    columns = np.linspace(datagrid.MinX, datagrid.MinX + datagrid.Dx * (datagrid.Nx - 1), datagrid.Nx)
    rows = np.linspace(datagrid.MinY, datagrid.MinY + datagrid.Dy * (datagrid.Ny - 1), datagrid.Ny)

    df = pd.DataFrame(data=values, index=rows[::-1], columns=columns)
    df.index.name = datagrid.YLabel if datagrid.YLabel is not None else "y"
    df.columns.name = datagrid.XLabel if datagrid.XLabel is not None else "x"

    return df


def standardize_sampling(sampling):
    """Standardizes the sampling patterns to either int or string (S_00x00) representation.

    Parameters
    ----------
    sampling: int or str
        The sampling pattern to use. Should be int or string. Accepts both S_00x00 and 00x00 string representation.

    Returns
    -------
    Union[int, str]
        The standardized sampling pattern that can be used for processing.

    Raises
    ------
    ValueError
        When 'sampling' is a string that cannot be interpreted
    TypeError
        When 'sampling' is not int or string
    """
    if isinstance(sampling, int):
        return sampling
    elif isinstance(sampling, str):
        res = re.search(r"\d+x\d+", sampling)
        if res:
            return "S_{}".format(res.group())
        else:
            raise ValueError('Cannot interpret sampling pattern "{}"'.format(sampling))
    else:
        raise TypeError("sampling should be int or string")



from __future__ import annotations

from dataclasses import dataclass

from zospy.api import _ZOSAPI, constants
from zospy.zpcore import OpticStudioSystem


@dataclass(frozen=True)
class PupilData:
    ApertureType: int
    ApertureValue: float
    EntrancePupilDiameter: float
    EntrancePupilPosition: float
    ExitPupilDiameter: float
    ExitPupilPosition: float
    ApodizationType: int
    ApodizationFactor: float


def get_pupil(oss: OpticStudioSystem):
    """Obtains the pupil data.

    Parameters
    ----------
    oss: zospy.core.OpticStudioSystem
        A ZOSPy OpticStudioSystem instance.

    Returns
    -------
    PupilData
        The pupildata.

    Examples
    --------
    >>> import zospy as zp
    >>> zos = zp.ZOS()
    >>> zos.connect_as_extension()
    >>> oss = zos.get_primary_system()
    >>> zp.functions.lde.get_pupil(oss)
    """
    return PupilData(*oss.LDE.GetPupil())


def surface_change_type(surface: _ZOSAPI.Editors.LDE.ILDERow, new_type: constants.Editors.LDE.SurfaceType | str):
    """Simple function to change the type of a surface in the LDE.

    Parameters
    ----------
    surface: ZOSAPI.Editors.LDE.ILDERow
        The Row/Surface for which the change is to be made.
    new_type: zospy.constants.Editors.LDE.SurfaceType | str
        The new surface type, either string (e.g. 'Standard') or int. The integer will be treated as if obtained from
        zp.constants.Editors.LDE.SurfaceType.

    Returns
    -------
    None

    Examples
    --------
    >>> import zospy as zp
    >>> zos = zp.ZOS()
    >>> zos.connect_as_extension()
    >>> oss = zos.get_primary_system()
    >>> newsurf = oss.LDE.InsertNewSurfaceAt(0)
    >>> zp.functions.lde.surface_change_type(newsurf, zp.constants.Editors.LDE.SurfaceType.Standard)
    """
    new_type = constants.process_constant(constants.Editors.LDE.SurfaceType, new_type)

    # Apply
    new_surface_type_settings = surface.GetSurfaceTypeSettings(new_type)
    surface.ChangeType(new_surface_type_settings)


def find_surface_by_comment(
    lde: _ZOSAPI.Editors.LDE, comment: str, case_sensitive: bool = True
) -> list[_ZOSAPI.Editors.LDE.ILDERow]:
    """Returns a list of surfaces from the LDE that have the supplied string as Comment.

    In case of multiple matches, the surfaces are returned in ascending order.

    Parameters
    ----------
    lde: ZOSAPI.Editors.LDE
        The Lens Data Editor (LDE)
    comment: str
        String that is searched for in the Comment column of the LDE.
    case_sensitive: bool
        Flag that specifies whether the search is case-sensitive or not. Defaults to True.

    Returns
    -------
    list[ZOSAPI.Editors.LDE.ILDERow]
        A list of surfaces in the LDE that have a Comment column value which matches the supplied comment argument.

    Examples
    --------
    >>> import zospy as zp
    >>> zos = zp.ZOS()
    >>> zos.wakeup()
    >>> zos.connect_as_extension()
    >>> oss = zos.get_primary_system()
    >>> newobj1 = oss.LDE.GetSurfaceAt(0)
    >>> newobj1.Comment = 'aa'
    >>> newobj2 = oss.LDE.GetSurfaceAt(1)
    >>> newobj2.Comment = 'bb'
    >>> newobj3 = oss.LDE.GetSurfaceAt(2)
    >>> newobj3.Comment = 'aA'
    >>> zp.functions.lde.find_surface_by_comment(oss.LDE, 'aa')
    """
    # Is the search case-sensitive?
    if not case_sensitive:
        # If the search is NOT case-sensitive put comment argument in all small letters
        comment = comment.lower()

    # Initialize list of return indexes corresponding to LDE rows with matched Comment column

    return_indices = []
    # Loop over the objects and check the comments

    for surface_index in range(lde.NumberOfSurfaces):
        # Retrieve current object comment
        surface = lde.GetSurfaceAt(surface_index)
        surface_comment = surface.Comment

        # Is the search case-sensitive?
        if not case_sensitive:
            # If the search is NOT case-sensitive put current object comment in all small letters
            surface_comment = surface_comment.lower()

        # If the comment matches, store the corresponding object index
        if comment == surface_comment:
            return_indices.append(surface)

    # Return list of matched inices
    return return_indices



"""
This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

from . import QuickYield

__all__ = ("QuickYield", "QYCompensations", "QYCompensatorStrategy", "QYPrecisions")

class QYCompensations(Enum):
    Standard = 0
    High = 1
    VeryHigh = 2

class QYCompensatorStrategy(Enum):
    OptimizeAllDampedLeastSquares = 0
    ParaxialFocus = 1
    Ignore = 2
    OptimizeAllOrthogonalDescent = 3

class QYPrecisions(Enum):
    Standard = 0
    High = 1
    VeryHigh = 2



"""OpticStudio Analyses.

Analyses in OpticStudio are available through `zospy.analyses`.
This module provides basic classes and functions for interacting with
analyses (in `zospy.analyses.base`), as well as a procedural interface
to several analyses.

Examples
--------
Run a Single Ray Trace analysis:

>>> from zospy.analyses.wavefront import zernike_standard_coefficients
>>> zernike_standard_coefficients(oss, sampling="32x32", maximum_term=15)

Open an analysis for which a wrapper function is not yet available:

>>> import zospy as zp
>>> analysis = zp.analyses.new_analysis(oss, zp.constants.Analysis.AnalysisIDM.ImageSimulation)
"""

from . import (
    mtf,
    polarization,
    psf,
    raysandspots,
    reports,
    surface,
    systemviewers,
    wavefront,
)
from .base import OnComplete, new_analysis

__all__ = (
    "mtf",
    "psf",
    "reports",
    "raysandspots",
    "polarization",
    "surface",
    "systemviewers",
    "wavefront",
    "new_analysis",
    "OnComplete",
)
